@@ -1,4 +1,5 @@
 #!/bin/sh
+# $Id: autogen.sh 3486 2006-09-21 00:58:22Z ckuethe $
 
 # Automakeversion
 AM_1=1
@@ -96,7 +97,7 @@ echo Configuring build environment for gpsd
 aclocal \
   && libtoolize --force --copy \
   && autoheader --force \
-  && automake --add-missing --foreign --copy \
+  && automake --add-missing --foreign --copy  --include-deps \
   && autoconf --force \
   && echo Now running configure to configure gpsd \
   && echo "./configure $@" \
