@@ -1,3 +1,4 @@
+/* $Id: geoid.c 3666 2006-10-26 23:11:51Z ckuethe $ */
 /* 
  * geoid.c -- ECEF to WGS84 conversions, including ellipsoid-to-MSL height
  *
@@ -5,7 +6,9 @@
  * ECEF conversion by Rob Janssen.
  */
 
+#include <sys/types.h>
 #include <math.h>
+#include "gpsd_config.h"
 #include "gpsd.h"
 
 static double bilinear(double x1, double y1, double x2, double y2, double x, double y, double z11, double z12, double z21, double z22)
@@ -92,20 +95,20 @@ void ecef_to_wgs84fix(struct gps_data_t *gpsdata,
     phi = atan2(z + e_2*b*pow(sin(theta),3),p - e2*a*pow(cos(theta),3));
     n = a / sqrt(1.0 - e2*pow(sin(phi),2));
     h = p / cos(phi) - n;
-    gpsdata->newdata.latitude = phi * RAD_2_DEG;
-    gpsdata->newdata.longitude = lambda * RAD_2_DEG;
-    gpsdata->separation = wgs84_separation(gpsdata->newdata.latitude, gpsdata->newdata.longitude);
-    gpsdata->newdata.altitude = h - gpsdata->separation;
+    gpsdata->fix.latitude = phi * RAD_2_DEG;
+    gpsdata->fix.longitude = lambda * RAD_2_DEG;
+    gpsdata->separation = wgs84_separation(gpsdata->fix.latitude, gpsdata->fix.longitude);
+    gpsdata->fix.altitude = h - gpsdata->separation;
     /* velocity computation */
     vnorth = -vx*sin(phi)*cos(lambda)-vy*sin(phi)*sin(lambda)+vz*cos(phi);
     veast = -vx*sin(lambda)+vy*cos(lambda);
-    gpsdata->newdata.climb = vx*cos(phi)*cos(lambda)+vy*cos(phi)*sin(lambda)+vz*sin(phi);
-    gpsdata->newdata.speed = sqrt(pow(vnorth,2) + pow(veast,2));
+    gpsdata->fix.climb = vx*cos(phi)*cos(lambda)+vy*cos(phi)*sin(lambda)+vz*sin(phi);
+    gpsdata->fix.speed = sqrt(pow(vnorth,2) + pow(veast,2));
     heading = atan2(veast,vnorth);
     /*@ +evalorder @*/
     if (heading < 0)
 	heading += 2 * PI;
-    gpsdata->newdata.track = heading * RAD_2_DEG;
+    gpsdata->fix.track = heading * RAD_2_DEG;
 }
 
 #ifdef TESTMAIN
