@@ -1,4 +1,6 @@
+/* $Id: gpsutils.c 4108 2006-12-08 11:48:31Z esr $ */
 /* gpsutils.c -- code shared between low-level and high-level interfaces */
+#include <sys/types.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -7,15 +9,17 @@
 #include <stdarg.h>
 #include <time.h>
 
+#include "gpsd_config.h"
 #include "gpsd.h"
 
 #define MONTHSPERYEAR	12		/* months per calendar year */
 
-void gps_clear_fix(/*@ out @*/struct gps_fix_t *fixp)
+void gps_clear_fix(/*@out@*/struct gps_fix_t *fixp)
 /* stuff a fix structure with recognizable out-of-band values */
 {
     fixp->time = NAN;
     fixp->mode = MODE_NOT_SEEN;
+    fixp->latitude = fixp->longitude = NAN;
     fixp->track = NAN;
     fixp->speed = NAN;
     fixp->climb = NAN;
@@ -28,6 +32,74 @@ void gps_clear_fix(/*@ out @*/struct gps_fix_t *fixp)
     fixp->epc = NAN;
 }
 
+unsigned int gps_valid_fields(/*@in@*/struct gps_fix_t *fixp)
+{
+    unsigned int valid = 0;
+
+    if (isnan(fixp->time) == 0)
+	valid |= TIME_SET;
+    if (fixp->mode != MODE_NOT_SEEN)
+	valid |= MODE_SET;
+    if (isnan(fixp->latitude) == 0 && isnan(fixp->longitude) == 0)
+	valid |= LATLON_SET;
+    if (isnan(fixp->altitude) == 0)
+	valid |= ALTITUDE_SET;
+    if (isnan(fixp->track) == 0)
+	valid |= TRACK_SET;
+    if (isnan(fixp->speed) == 0)
+	valid |= SPEED_SET;
+    if (isnan(fixp->climb) == 0)
+	valid |= CLIMB_SET;
+    if (isnan(fixp->ept) == 0)
+	valid |= TIMERR_SET;
+    if (isnan(fixp->eph) == 0)
+	valid |= HERR_SET;
+    if (isnan(fixp->epv) == 0)
+	valid |= VERR_SET;
+    if (isnan(fixp->epd) == 0)
+	valid |= TRACKERR_SET;
+    if (isnan(fixp->eps) == 0)
+	valid |= SPEEDERR_SET;
+    if (isnan(fixp->epc) == 0)
+	valid |= CLIMBERR_SET;
+    return valid;
+}
+
+char *gps_show_transfer(int transfer)
+{
+/*@ -statictrans @*/
+    static char showbuf[100];
+    showbuf[0] = '\0';
+    if ((transfer & TIME_SET)!=0)
+	(void)strlcat(showbuf, "time,", sizeof(showbuf));
+    if ((transfer & LATLON_SET)!=0)
+	(void)strlcat(showbuf, "latlon,", sizeof(showbuf));
+    if ((transfer & MODE_SET)!=0)
+	(void)strlcat(showbuf, "mode,", sizeof(showbuf));
+    if ((transfer & ALTITUDE_SET)!=0)
+	(void)strlcat(showbuf, "altitude,", sizeof(showbuf));
+    if ((transfer & TRACK_SET)!=0)
+	(void)strlcat(showbuf, "track,", sizeof(showbuf));
+    if ((transfer & SPEED_SET)!=0)
+	(void)strlcat(showbuf, "speed,", sizeof(showbuf));
+    if ((transfer & CLIMB_SET)!=0)
+	(void)strlcat(showbuf, "climb,", sizeof(showbuf));
+    if ((transfer & TIMERR_SET)!=0)
+	(void)strlcat(showbuf, "timerr,", sizeof(showbuf));
+    if ((transfer & HERR_SET)!=0)
+	(void)strlcat(showbuf, "herr,", sizeof(showbuf));
+    if ((transfer & VERR_SET)!=0)
+	(void)strlcat(showbuf, "verr,", sizeof(showbuf));
+    if ((transfer & SPEEDERR_SET)!=0)
+	(void)strlcat(showbuf, "speederr,", sizeof(showbuf));
+    if ((transfer & CLIMBERR_SET)!=0)
+	(void)strlcat(showbuf, "climberr,", sizeof(showbuf));
+    if (strlen(showbuf)>0)
+	showbuf[strlen(showbuf)-1] = '\0';
+    return showbuf;
+/*@ +statictrans @*/
+}
+
 void gps_merge_fix(/*@ out @*/struct gps_fix_t *to,
 		   gps_mask_t transfer,
 		   /*@ in @*/struct gps_fix_t *from)
@@ -126,7 +198,7 @@ double iso8601_to_unix(/*@in@*/char *isotime)
     /*@ -aliasunique @*/
     (void)memcpy(isotime+slen, isotime+slen+1, strlen(isotime+slen+1));
     /*@ -aliasunique @*/
-    (void)strcat(isotime, "Z");
+    (void)strlcat(isotime, "Z", 28);
     return isotime;
 }
 
@@ -210,13 +282,13 @@ double earth_distance(double lat1, double lon1, double lat2, double lon2)
     double a = (x1*x2 + y1*y2 + z1*z2)/pow(CalcRad((lat1+lat2)/2),2);
     // a should be in [1, -1] but can sometimes fall outside it by
     // a very small amount due to rounding errors in the preceding
-    // calculations (this is prone to happen when the argument points
-    // are very close together).  Thus we constrain it here.
+    // calculations.  This is prone to happen when the argument points
+    // are very close together.  Return NaN so calculations trying
+    // to use this will also blow up.
     if (fabs(a) > 1) 
-	a = 1;
-    else if (a < -1) 
-	a = -1;
-    return CalcRad((lat1+lat2) / 2) * acos(a);
+	return NAN;
+    else
+	return CalcRad((lat1+lat2) / 2) * acos(a);
 }
 
 /*****************************************************************************
@@ -371,8 +443,8 @@ static int invert(double mat[4][4], /*@out@*/double inverse[4][4])
 		+ mat[0][2]*Det3_123_013 
 		- mat[0][3]*Det3_123_012;
 
-  // ??? Should the test be made: fabs(det) <= epsilon ???
-  if (det == 0.0)
+  // Very small determinants probably reflect floating-point fuzz near zero
+  if (fabs(det) == 0.0)
       return 0;
 
   inverse[0][0] =  Det3_123_123 / det;
@@ -407,16 +479,16 @@ gps_mask_t dop(struct gps_data_t *gpsdata)
     int i, j, k, n;
 
 #ifdef __UNUSED__
-    gpsd_report(0, "Satellite picture:\n");
-    for (k = 0; k < gpsdata->device_type.channels; k++) {
+    gpsd_report(LOG_INF, "Satellite picture:\n");
+    for (k = 0; k < MAXCHANNELS; k++) {
 	if (gpsdata->used[k])
-	    gpsd_report(0, "az: %d el: %d  SV: %d\n",
+	    gpsd_report(LOG_INF, "az: %d el: %d  SV: %d\n",
 			gpsdata->azimuth[k], gpsdata->elevation[k], gpsdata->used[k]);
     }
 #endif /* __UNUSED__ */
 
     for (n = k = 0; k < gpsdata->satellites_used; k++) {
-	if (gpsdata->used[k] != 0)
+	if (gpsdata->used[k] == 0)
 	    continue;
 	satpos[n][0] = sin(gpsdata->azimuth[k]*DEG_2_RAD)
 	    * cos(gpsdata->elevation[k]*DEG_2_RAD);
@@ -428,9 +500,9 @@ gps_mask_t dop(struct gps_data_t *gpsdata)
     }
 
 #ifdef __UNUSED__
-    gpsd_report(0, "Line-of-sight matrix:\n");
+    gpsd_report(LOG_INF, "Line-of-sight matrix:\n");
     for (k = 0; k < n; k++) {
-	gpsd_report(0, "%f %f %f %f\n",
+	gpsd_report(LOG_INF, "%f %f %f %f\n",
 		    satpos[k][0], satpos[k][1], satpos[k][2], satpos[k][3]);
     }
 #endif /* __UNUSED__ */
@@ -445,25 +517,29 @@ gps_mask_t dop(struct gps_data_t *gpsdata)
     }
 
 #ifdef __UNUSED__
-    gpsd_report(0, "product:\n");
+    gpsd_report(LOG_INF, "product:\n");
     for (k = 0; k < 4; k++) {
-	gpsd_report(0, "%f %f %f %f\n",
+	gpsd_report(LOG_INF, "%f %f %f %f\n",
 		    prod[k][0], prod[k][1], prod[k][2], prod[k][3]);
     }
 #endif /* __UNUSED__ */
 
     if (invert(prod, inv)) {
 #ifdef __UNUSED__
-	gpsd_report(0, "inverse:\n");
+	/*
+	 * Note: this will print garbage unless all the subdeterminants
+	 * are computed in the invert() function.
+	 */
+	gpsd_report(LOG_RAW, "inverse:\n");
 	for (k = 0; k < 4; k++) {
-	    gpsd_report(0, "%f %f %f %f\n",
+	    gpsd_report(LOG_RAW, "%f %f %f %f\n",
 			inv[k][0], inv[k][1], inv[k][2], inv[k][3]);
 	}
-	gpsd_report(1, "HDOP: reported = %f, computed = %f\n",
+	gpsd_report(LOG_INF, "HDOP: reported = %f, computed = %f\n",
 		    gpsdata->hdop, sqrt(inv[0][0] + inv[1][1]));
 #endif /* __UNUSED__ */
     } else {
-	gpsd_report(1, "LOS matrix is singular, can't calculate DOPs.\n");
+	gpsd_report(LOG_WARN, "LOS matrix is singular, can't calculate DOPs.\n");
 	return 0;
     }
 
