@@ -1,3 +1,5 @@
+/* $Id: gpxlogger.c 3666 2006-10-26 23:11:51Z ckuethe $ */
+#include <sys/types.h>
 #include <string.h>
 #include <stdlib.h>
 #include <syslog.h>
@@ -11,11 +13,12 @@
 #include <dbus/dbus-glib.h>
 
 #include <glib/gprintf.h>
+#include "gpsd_config.h"
 
 DBusConnection* connection;
 
 static char *author = "Amaury Jacquot";
-static char *copyright = "GPL v 2.0";
+static char *copyright = "BSD or GPL v 2.0";
 
 static int intrack = 0;
 static int first = 1;
@@ -54,7 +57,6 @@ static void print_gpx_trk_end (void) {
 }
 
 static DBusHandlerResult handle_gps_fix (DBusMessage* message) {
-	DBusMessageIter	iter;
 	DBusError	error;
 	double		temp_time;
 
@@ -170,21 +172,15 @@ int main (int argc, char** argv) {
 	DBusError error;
 
 	/* initializes the gpsfix data structure */
-	bzero (&gpsfix, sizeof(gpsfix));
+	gps_clear_fix(&gpsfix);
 
 	/* catch all interesting signals */
 	signal (SIGTERM, quit_handler);
 	signal (SIGQUIT, quit_handler);
 	signal (SIGINT, quit_handler);
-			
-	
-	openlog ("gpxlogger", LOG_PID | LOG_NDELAY , LOG_DAEMON);
-	syslog (LOG_INFO, "---------- STARTED ----------");
-	
-	if (argc<2) {
-		fprintf (stderr, "need the filename as an argument\n");
-		return 1;
-	}
+
+	//openlog ("gpxlogger", LOG_PID | LOG_NDELAY , LOG_DAEMON);
+	//syslog (LOG_INFO, "---------- STARTED ----------");
 	
 	print_gpx_header ();
 	
