@@ -1,15 +1,18 @@
+/* $Id: libgps.c 4070 2006-12-04 12:44:33Z esr $ */
 /* libgps.c -- client interface library for the gpsd daemon */
+#include <sys/time.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <sys/time.h>
+#include <stdarg.h>
 #ifndef S_SPLINT_S
 #include <pthread.h>	/* pacifies OpenBSD's compiler */
 #endif
 #include <math.h>
 
+#include "gpsd_config.h"
 #include "gpsd.h"
 
 #ifdef S_SPLINT_S
@@ -32,7 +35,7 @@ extern char *strtok_r(char *, const char *, char **);
     double fdsec, fsec, fdeg, fmin;
 
     if ( f < 0 || f > 360 ) {
-	strcpy( str, "nan");
+	(void)strlcpy( str, "nan", 40);
 	return str;
     }
 
@@ -224,14 +227,18 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    }
 		    break;
 		case 'E':
-		    if (sp[2] == '?') {
-			   gpsdata->epe = NAN;
-			   gpsdata->fix.eph = NAN;
-			   gpsdata->fix.epv = NAN;
-		    } else {
-		        (void)sscanf(sp, "E=%lf %lf %lf", 
-			   &gpsdata->epe,&gpsdata->fix.eph,&gpsdata->fix.epv);
-		        gpsdata->set |= HERR_SET| VERR_SET | PERR_SET;
+		    gpsdata->epe = gpsdata->fix.eph = gpsdata->fix.epv = NAN;
+		    /* epe should always be present if eph or epv is */
+		    if (sp[2] != '?') {
+			char epe[20], eph[20], epv[20];
+		        (void)sscanf(sp, "E=%s %s %s", epe, eph, epv);
+#define DEFAULT(val) (val[0] == '?') ? NAN : atof(val)
+			    /*@ +floatdouble @*/
+			    gpsdata->epe = DEFAULT(epe);
+			    gpsdata->fix.eph = DEFAULT(eph);
+			    gpsdata->fix.epv = DEFAULT(epv);
+			    /*@ -floatdouble @*/
+#undef DEFAULT
 		    }
 		    break;
 		case 'F':
@@ -246,15 +253,6 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    }
 		    /*@ +mustfreeonly */
 		    break;
-#ifdef HEADING_FIX
-		case 'H':
-		    if (sp[2] == '?') {
-		        gpsdata->fix.heading = NAN;
-		    } else {
-		        (void)sscanf(sp, "H=%lf", &gpsdata->fix.heading);
-		    }
-		    break;
-#endif /* HEADING_FIX */
 		case 'I':
 		    /*@ -mustfreeonly */
 		    if (gpsdata->gps_id)
@@ -315,15 +313,19 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			char tag[MAXTAGLEN+1], alt[20];
 			char eph[20], epv[20], track[20],speed[20], climb[20];
 			char epd[20], eps[20], epc[20], mode[2];
+			char timestr[20], ept[20], lat[20], lon[20];
 			int st = sscanf(sp+2, 
-			       "%8s %lf %lf %lf %lf %s %s %s %s %s %s %s %s %s %s",
-				tag, &nf.time, &nf.ept, 
-				&nf.latitude, &nf.longitude,
+			       "%8s %19s %19s %19s %19s %19s %19s %19s %19s %19s %19s %19s %19s %19s %1s",
+				tag, timestr, ept, lat, lon,
 			        alt, eph, epv, track, speed, climb,
 			        epd, eps, epc, mode);
 			if (st == 15) {
 #define DEFAULT(val) (val[0] == '?') ? NAN : atof(val)
 			    /*@ +floatdouble @*/
+			    nf.time = DEFAULT(timestr);
+			    nf.latitude = DEFAULT(lat);
+			    nf.longitude = DEFAULT(lon);
+			    nf.ept = DEFAULT(ept);
 			    nf.altitude = DEFAULT(alt);
 			    nf.eph = DEFAULT(eph);
 			    nf.epv = DEFAULT(epv);
@@ -351,14 +353,8 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 				gpsdata->set |= SPEEDERR_SET;
 			    if (isnan(nf.epc)==0)
 				gpsdata->set |= CLIMBERR_SET;
-#ifndef HEADING_FIX
-			    nf.pitch = nf.roll = nf.dip = NAN;
-#else /* HEADING_FIX */
-			    /* there is no support for a TrueNorth source in the O command at this stage, so libgps-based clients that wish to use the heading provided by such a source must send the sequence "OH\n".  N.B. the order is important - "HO\n" will result in heading=NAN.  More work is needed to provide any support for pitch, roll and dip. */
-			    nf.pitch = nf.roll = nf.dip = nf.heading = NAN;
-#endif /* HEADING_FIX */
 			    gpsdata->fix = nf;
-			    (void)strcpy(gpsdata->tag, tag);
+			    (void)strlcpy(gpsdata->tag, tag, MAXTAGLEN+1);
 			    gpsdata->set |= TIME_SET|TIMERR_SET|LATLON_SET|MODE_SET;
 			    gpsdata->status = STATUS_FIX;
 			    gpsdata->set |= STATUS_SET;
@@ -442,9 +438,9 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			int PRN[MAXCHANNELS];
 			int elevation[MAXCHANNELS], azimuth[MAXCHANNELS];
 			int ss[MAXCHANNELS], used[MAXCHANNELS];
-			char tag[21], timestamp[21];
+			char tag[MAXTAGLEN+1], timestamp[21];
 
-			(void)sscanf(sp, "Y=%20s %20s %d ", 
+			(void)sscanf(sp, "Y=%8s %20s %d ", 
 			       tag, timestamp, &gpsdata->satellites);
 			(void)strncpy(gpsdata->tag, tag, MAXTAGLEN);
 			if (timestamp[0] != '?') {
@@ -455,13 +451,15 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 			    PRN[j]=elevation[j]=azimuth[j]=ss[j]=used[j]=0;
 			}
 			for (j = 0, gpsdata->satellites_used = 0; j < gpsdata->satellites; j++) {
-			    sp = strchr(sp, ':') + 1;
-			    (void)sscanf(sp, "%d %d %d %d %d", &i1, &i2, &i3, &i4, &i5);
-			    PRN[j] = i1;
-			    elevation[j] = i2; azimuth[j] = i3;
-			    ss[j] = i4; used[j] = i5;
-			    if (i5 == 1)
-                                gpsdata->satellites_used++;
+			    if ((sp != NULL) && ((sp = strchr(sp, ':')) != NULL)) {
+				sp++;
+				(void)sscanf(sp, "%d %d %d %d %d", &i1, &i2, &i3, &i4, &i5);
+				PRN[j] = i1;
+				elevation[j] = i2; azimuth[j] = i3;
+				ss[j] = i4; used[j] = i5;
+				if (i5 == 1)
+				    gpsdata->satellites_used++;
+			    }
 			}
 			/*@ -compdef @*/
 			memcpy(gpsdata->PRN, PRN, sizeof(PRN));
@@ -477,8 +475,10 @@ static void gps_unpack(char *buf, struct gps_data_t *gpsdata)
 		    gpsdata->profiling = (sp[2] == '1');
 		    break;
 		case '$':
+		    if (gpsdata->profiling != true)
+			break;
 		    /*@ +matchanyintegral -formatcode @*/
-		    (void)sscanf(sp, "$=%s %zd %lf %lf %lf %lf %lf %lf", 
+		    (void)sscanf(sp, "$=%8s %zd %lf %lf %lf %lf %lf %lf", 
 			   gpsdata->tag,
 			   &gpsdata->sentence_length,
 			   &gpsdata->fix.time, 
@@ -533,10 +533,18 @@ int gps_poll(struct gps_data_t *gpsdata)
     return 0;
 }
 
-int gps_query(struct gps_data_t *gpsdata, const char *requests)
+int gps_query(struct gps_data_t *gpsdata, const char *fmt, ... )
 /* query a gpsd instance for new data */
 {
-    if (write(gpsdata->gps_fd, requests, strlen(requests)) <= 0)
+    char buf[BUFSIZ];
+    va_list ap;
+
+    va_start(ap, fmt);
+    (void)vsnprintf(buf, sizeof(buf)-2, fmt, ap);
+    va_end(ap);
+    if (buf[strlen(buf)-1] != '\n')
+	(void)strlcat(buf, "\n", BUFSIZ);
+    if (write(gpsdata->gps_fd, buf, strlen(buf)) <= 0)
 	return -1;
     return gps_poll(gpsdata);
 }
@@ -662,8 +670,8 @@ int main(int argc, char *argv[])
     collect = gps_open(NULL, 0);
     gps_set_raw_hook(collect, dumpline);
     if (optind < argc) {
-	strcpy(buf, argv[optind]);
-	strcat(buf,"\n");
+	strlcpy(buf, argv[optind], BUFSIZ);
+	strlcat(buf,"\n", BUFSIZ);
 	gps_query(collect, buf);
 	data_dump(collect, time(NULL));
     } else {
