@@ -1,23 +1,16 @@
+/* $Id: libgpsmm.h 3666 2006-10-26 23:11:51Z ckuethe $ */
 /*
  * Copyright (C) 2005 Alfredo Pironti
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * This software is distributed under a BSD-style license. See the
+ * file "COPYING" for more information.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 #ifndef _GPSMM_H_
 #define _GPSMM_H_
 
+#include <sys/types.h>
+#include "gpsd_config.h"
 #include "gps.h" //the C library we are going to wrap
 
 class gpsmm {
