@@ -1,3 +1,5 @@
+/* $Id: nmea_parse.c 4027 2006-11-29 12:37:08Z esr $ */
+#include <sys/types.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -7,6 +9,7 @@
 #include <ctype.h>
 #include <time.h>
 
+#include "gpsd_config.h"
 #include "gpsd.h"
 #include "timebase.h"
 
@@ -32,8 +35,8 @@ static void do_lat_lon(char *field[], struct gps_data_t *out)
 	p = field[1];
 	if (*p == 'S')
 	    lat = -lat;
-	if (out->newdata.latitude != lat)
-	    out->newdata.latitude = lat;
+	if (out->fix.latitude != lat)
+	    out->fix.latitude = lat;
 	updated++;
     }
     if (*(p = field[2]) != '\0') {
@@ -45,8 +48,8 @@ static void do_lat_lon(char *field[], struct gps_data_t *out)
 	p = field[3];
 	if (*p == 'W')
 	    lon = -lon;
-	if (out->newdata.longitude != lon)
-	    out->newdata.longitude = lon;
+	if (out->fix.longitude != lon)
+	    out->fix.longitude = lon;
 	updated++;
     }
 }
@@ -126,8 +129,8 @@ static gps_mask_t processGPRMC(int count, char *field[], struct gps_device_t *se
 	    session->gpsdata.status = STATUS_NO_FIX;
 	    mask |= STATUS_SET;
 	}
-	if (session->gpsdata.newdata.mode >= MODE_2D) {
-	    session->gpsdata.newdata.mode = MODE_NO_FIX;
+	if (session->gpsdata.fix.mode >= MODE_2D) {
+	    session->gpsdata.fix.mode = MODE_NO_FIX;
 	    mask |= MODE_SET;
 	}
 	/* set something nz, so it won't look like an unknown sentence */
@@ -137,28 +140,28 @@ static gps_mask_t processGPRMC(int count, char *field[], struct gps_device_t *se
 	    merge_ddmmyy(field[9], session);
 	    merge_hhmmss(field[1], session);
 	    mask |= TIME_SET;
-	    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
-	    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+	    session->gpsdata.fix.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
+	    if (session->gpsdata.sentence_time != session->gpsdata.fix.time)
 		mask |= CYCLE_START_SET;
-	    session->gpsdata.sentence_time = session->gpsdata.newdata.time;
+	    session->gpsdata.sentence_time = session->gpsdata.fix.time;
 	}
 	do_lat_lon(&field[3], &session->gpsdata);
 	mask |= LATLON_SET;
-	session->gpsdata.newdata.speed = atof(field[7]) * KNOTS_TO_MPS;
-	session->gpsdata.newdata.track = atof(field[8]);
+	session->gpsdata.fix.speed = atof(field[7]) * KNOTS_TO_MPS;
+	session->gpsdata.fix.track = atof(field[8]);
 	mask |= (TRACK_SET | SPEED_SET);
 	/*
 	 * This copes with GPSes like the Magellan EC-10X that *only* emit
 	 * GPRMC. In this case we set mode and status here so the client
 	 * code that relies on them won't mistakenly believe it has never
-	 * received a newdata.
+	 * received a fix.
 	 */
 	if (session->gpsdata.status == STATUS_NO_FIX) {
 	    session->gpsdata.status = STATUS_FIX;	/* could be DGPS_FIX, we can't tell */
 	    mask |= STATUS_SET;
 	}
-	if (session->gpsdata.newdata.mode < MODE_2D) {
-	    session->gpsdata.newdata.mode = MODE_2D;
+	if (session->gpsdata.fix.mode < MODE_2D) {
+	    session->gpsdata.fix.mode = MODE_2D;
 	    mask |= MODE_SET;
 	}
     }
@@ -200,14 +203,14 @@ static gps_mask_t processGPGLL(int count, char *field[], struct gps_device_t *se
 
 	mask = 0;
 	merge_hhmmss(field[5], session);
-	if (session->driver.nmea.date.tm_year != 0) 
-	    gpsd_report(1, "can't use GGA/GGL time until after ZDA or RMC has supplied a year.\n");
+	if (session->driver.nmea.date.tm_year == 0) 
+	    gpsd_report(LOG_WARN, "can't use GGA/GGL time until after ZDA or RMC has supplied a year.\n");
 	else {
 	    mask = TIME_SET;
-	    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
-	    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+	    session->gpsdata.fix.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
+	    if (session->gpsdata.sentence_time != session->gpsdata.fix.time)
 		mask |= CYCLE_START_SET;
-	    session->gpsdata.sentence_time = session->gpsdata.newdata.time;
+	    session->gpsdata.sentence_time = session->gpsdata.fix.time;
 	}
 	do_lat_lon(&field[1], &session->gpsdata);
 	mask |= LATLON_SET;
@@ -215,9 +218,20 @@ static gps_mask_t processGPGLL(int count, char *field[], struct gps_device_t *se
 	    newstatus = STATUS_DGPS_FIX;	/* differential */
 	else
 	    newstatus = STATUS_FIX;
+	/*
+	 * This is a bit dodgy.  Technically we shouldn't set the mode 
+	 * bit until we see GSA.  But it may be later in the cycle,
+	 * some devices like the FV-18 don't send it by default, and 
+	 * elsewhere in the code we want to be able to test for the 
+	 * presence of a valid fix with mode > MODE_NO_FIX.
+	 */
+	if (session->gpsdata.fix.mode < MODE_2D) {
+	    session->gpsdata.fix.mode = MODE_2D;
+	    mask |= MODE_SET;
+	}
 	session->gpsdata.status = newstatus;
 	mask |= STATUS_SET;
-	gpsd_report(3, "GPGLL sets status %d\n", session->gpsdata.status);
+	gpsd_report(LOG_PROG, "GPGLL sets status %d\n", session->gpsdata.status);
     }
 
     return mask;
@@ -246,21 +260,21 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[], struct gps_device_t
     gps_mask_t mask;
 
     session->gpsdata.status = atoi(field[6]);
-    gpsd_report(3, "GPGGA sets status %d\n", session->gpsdata.status);
+    gpsd_report(LOG_PROG, "GPGGA sets status %d\n", session->gpsdata.status);
     mask = STATUS_SET;
     if (session->gpsdata.status > STATUS_NO_FIX) {
 	char *altitude;
-	double oldfixtime = session->gpsdata.newdata.time;
+	double oldfixtime = session->gpsdata.fix.time;
 
 	merge_hhmmss(field[1], session);
 	if (session->driver.nmea.date.tm_year == 0) 
-	    gpsd_report(1, "can't use GGA/GGL time until after ZDA or RMC has supplied a year.\n");
+	    gpsd_report(LOG_WARN, "can't use GGA/GGL time until after ZDA or RMC has supplied a year.\n");
 	else {
 	    mask |= TIME_SET;
-	    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
-	    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+	    session->gpsdata.fix.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
+	    if (session->gpsdata.sentence_time != session->gpsdata.fix.time)
 		mask |= CYCLE_START_SET;
-	    session->gpsdata.sentence_time = session->gpsdata.newdata.time;
+	    session->gpsdata.sentence_time = session->gpsdata.fix.time;
 	}
 	do_lat_lon(&field[2], &session->gpsdata);
 	mask |= LATLON_SET;
@@ -272,16 +286,26 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[], struct gps_device_t
 	 * If we see this, force mode to 2D at most.
 	 */
 	if (altitude[0] == '\0') {
-	    if (session->gpsdata.newdata.mode == MODE_3D) {
-		session->gpsdata.newdata.mode = session->gpsdata.status ? MODE_2D : MODE_NO_FIX; 
+	    if (session->gpsdata.fix.mode == MODE_3D) {
+		session->gpsdata.fix.mode = session->gpsdata.status ? MODE_2D : MODE_NO_FIX; 
 		mask |= MODE_SET;
 	    }
 	} else {
-	    double oldaltitude = session->gpsdata.newdata.altitude;
+	    double oldaltitude = session->gpsdata.fix.altitude;
 
-	    session->gpsdata.newdata.altitude = atof(altitude);
+	    session->gpsdata.fix.altitude = atof(altitude);
 	    mask |= ALTITUDE_SET;
-
+	    /*
+	     * This is a bit dodgy.  Technically we shouldn't set the mode 
+	     * bit until we see GSA.  But it may be later in the cycle,
+	     * some devices like the FV-18 don't send it by default, and 
+	     * elsewhere in the code we want to be able to test for the 
+	     * presence of a valid fix with mode > MODE_NO_FIX.
+	     */
+	    if (session->gpsdata.fix.mode < MODE_3D) {
+		session->gpsdata.fix.mode = MODE_3D;
+		mask |= MODE_SET;
+	    }
 
 	    /*
 	     * Compute climb/sink in the simplest possible way.
@@ -289,17 +313,17 @@ static gps_mask_t processGPGGA(int c UNUSED, char *field[], struct gps_device_t
 	     * SiRF and Garmin chips, which might have some smoothing
 	     * going on.
 	     */
-	    if (isnan(oldaltitude)==0 || session->gpsdata.newdata.time==oldfixtime)
-		session->gpsdata.newdata.climb = 0;
+	    if (isnan(oldaltitude)!=0 || session->gpsdata.fix.time==oldfixtime)
+		session->gpsdata.fix.climb = 0;
 	    else {
-		session->gpsdata.newdata.climb = (session->gpsdata.newdata.altitude-oldaltitude)/(session->gpsdata.newdata.time-oldfixtime);
+		session->gpsdata.fix.climb = (session->gpsdata.fix.altitude-oldaltitude)/(session->gpsdata.fix.time-oldfixtime);
 	    }
 	    mask |= CLIMB_SET;
 	}
 	if (strlen(field[11]) > 0) {
 	   session->gpsdata.separation = atof(field[11]);
 	} else {
-	   session->gpsdata.separation = wgs84_separation(session->gpsdata.newdata.latitude,session->gpsdata.newdata.longitude);
+	   session->gpsdata.separation = wgs84_separation(session->gpsdata.fix.latitude,session->gpsdata.fix.longitude);
 	}
     }
     return mask;
@@ -328,28 +352,29 @@ static gps_mask_t processGPGSA(int count, char *field[], struct gps_device_t *se
      * this: "$GPGSA,A,1,,,,*32" when it has no fix.  This is broken
      * in at least two ways: it's got the wrong number of fields, and
      * it claims to be a valid sentence (A flag) when it isn't.
-     * Alarmingly, it's possible this error may be generic to SiRF-IIIs.
+     * Alarmingly, it's possible this error may be generic to SiRFstarIII.
      */
     if (count < 17)
 	return ONLINE_SET;
 
-    session->gpsdata.newdata.mode = atoi(field[2]);
+    session->gpsdata.fix.mode = atoi(field[2]);
     /*
      * The first arm of this conditional ignores dead-reckoning
      * fixes from an Antaris chipset. which returns E in field 2
      * for a dead-reckoning estimate.  Fix by Andreas Stricker.
      */
-    if (session->gpsdata.newdata.mode == 0 && field[2][0] == 'E')
+    if (session->gpsdata.fix.mode == 0 && field[2][0] == 'E')
         mask = 0;
     else
         mask = MODE_SET;
-    gpsd_report(3, "GPGSA sets mode %d\n", session->gpsdata.newdata.mode);
-    session->gpsdata.pdop = atof(field[session->device_type->channels+3]);
-    session->gpsdata.hdop = atof(field[session->device_type->channels+4]);
-    session->gpsdata.vdop = atof(field[session->device_type->channels+5]);
+    gpsd_report(LOG_PROG, "GPGSA sets mode %d\n", session->gpsdata.fix.mode);
+    session->gpsdata.pdop = atof(field[count-3]);
+    session->gpsdata.hdop = atof(field[count-2]);
+    session->gpsdata.vdop = atof(field[count-1]);
     session->gpsdata.satellites_used = 0;
     memset(session->gpsdata.used,0,sizeof(session->gpsdata.used));
-    for (i = 0; i < session->device_type->channels; i++) {
+    /* the magic 6 here counts the tag, two mode fields, and the DOP fields */
+    for (i = 0; i < count - 6; i++) {
         int prn = atoi(field[i+3]);
         if (prn > 0)
 	    session->gpsdata.used[session->gpsdata.satellites_used++] = prn;
@@ -387,8 +412,8 @@ static gps_mask_t processGPGSV(int count, char *field[], struct gps_device_t *se
 	gpsd_zero_satellites(&session->gpsdata);
 
     for (fldnum = 4; fldnum < count; ) {
-	if (session->gpsdata.satellites >= session->device_type->channels) {
-	    gpsd_report(0, "internal error - too many satellites!\n");
+	if (session->gpsdata.satellites >= MAXCHANNELS) {
+	    gpsd_report(LOG_ERROR, "internal error - too many satellites!\n");
 	    gpsd_zero_satellites(&session->gpsdata);
 	    break;
 	}
@@ -406,16 +431,16 @@ static gps_mask_t processGPGSV(int count, char *field[], struct gps_device_t *se
 	    session->gpsdata.satellites++;
     }
     if (session->driver.nmea.part == session->driver.nmea.await && atoi(field[3]) != session->gpsdata.satellites)
-	gpsd_report(0, "GPGSV field 3 value of %d != actual count %d\n",
+	gpsd_report(LOG_WARN, "GPGSV field 3 value of %d != actual count %d\n",
 		    atoi(field[3]), session->gpsdata.satellites);
 
     /* not valid data until we've seen a complete set of parts */
     if (session->driver.nmea.part < session->driver.nmea.await) {
-	gpsd_report(3, "Partial satellite data (%d of %d).\n", session->driver.nmea.part, session->driver.nmea.await);
+	gpsd_report(LOG_PROG, "Partial satellite data (%d of %d).\n", session->driver.nmea.part, session->driver.nmea.await);
 	return ERROR_SET;
     }
     /*
-     * This sanity check catches an odd behavior of SiRF-II based GPSes.
+     * This sanity check catches an odd behavior of SiRFstarII receivers.
      * When they can't see any satellites at all (like, inside a
      * building) they sometimes cough up a hairball in the form of a
      * GSV packet with all the azimuth entries 0 (but nonzero
@@ -425,11 +450,11 @@ static gps_mask_t processGPGSV(int count, char *field[], struct gps_device_t *se
     for (n = 0; n < session->gpsdata.satellites; n++)
 	if (session->gpsdata.azimuth[n] != 0)
 	    goto sane;
-    gpsd_report(3, "Satellite data no good.\n");
+    gpsd_report(LOG_WARN, "Satellite data no good.\n");
     gpsd_zero_satellites(&session->gpsdata);
     return ERROR_SET;
   sane:
-    gpsd_report(3, "Satellite data OK.\n");
+    gpsd_report(LOG_PROG, "Satellite data OK.\n");
     return SATELLITE_SET;
     }
 
@@ -447,10 +472,22 @@ static gps_mask_t processPGRME(int c UNUSED, char *field[], struct gps_device_t
      *
      * Garmin won't say, but the general belief is that these are 50% CEP.
      * We follow the advice at <http://gpsinformation.net/main/errors.htm>.
+     * If this assumption changes here, it should also change in garmin.c
+     * where we scale error estimates from Garmin binary packets.
      */
-    session->gpsdata.newdata.eph = atof(field[1]) * (GPSD_CONFIDENCE/CEP50_SIGMA);
-    session->gpsdata.newdata.epv = atof(field[3]) * (GPSD_CONFIDENCE/CEP50_SIGMA);
+    if ((strcmp(field[2], "M")!=0) ||
+	(strcmp(field[4], "M")!=0) ||
+	(strcmp(field[6], "M")!=0)){
+	    session->gpsdata.fix.eph =
+	    session->gpsdata.fix.epv =
+	    session->gpsdata.epe = 100;
+	    return ERROR_SET;
+    }
+
+    session->gpsdata.fix.eph = atof(field[1]) * (GPSD_CONFIDENCE/CEP50_SIGMA);
+    session->gpsdata.fix.epv = atof(field[3]) * (GPSD_CONFIDENCE/CEP50_SIGMA);
     session->gpsdata.epe = atof(field[5]) * (GPSD_CONFIDENCE/CEP50_SIGMA);
+
     return HERR_SET | VERR_SET | PERR_SET;
 }
 
@@ -472,10 +509,10 @@ static gps_mask_t processGPZDA(int c UNUSED, char *field[], struct gps_device_t
     session->driver.nmea.date.tm_year = atoi(field[4]) - 1900;
     session->driver.nmea.date.tm_mon = atoi(field[3])-1;
     session->driver.nmea.date.tm_mday = atoi(field[2]);
-    session->gpsdata.newdata.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
-    if (session->gpsdata.sentence_time != session->gpsdata.newdata.time)
+    session->gpsdata.fix.time = (double)mkgmtime(&session->driver.nmea.date)+session->driver.nmea.subseconds;
+    if (session->gpsdata.sentence_time != session->gpsdata.fix.time)
 	mask |= CYCLE_START_SET;
-    session->gpsdata.sentence_time = session->gpsdata. newdata.time;
+    session->gpsdata.sentence_time = session->gpsdata. fix.time;
     return mask;
 }
 
@@ -511,53 +548,26 @@ static gps_mask_t processTNTHTM(int c UNUSED, char *field[], struct gps_device_t
 
     //gpsd_zero_satellites(&session->gpsdata);
 
-#ifdef HEADING_FIX
-	// these fields are not handled by gps_merge_fix(), so must be written directly to the current fix structure
-#endif /* HEADING_FIX */
+    /*
+     * Heading maps to track.  
+     * Pitch maps to climb.  
+     * Roll maps to speed.
+     * Dip maps to altitude.
+     */
     session->gpsdata.fix.time = timestamp();
-#ifndef HEADING_FIX
-    session->gpsdata.newdata.track = atof(field[1]);
-#else /* HEADING_FIX */
-    session->gpsdata.fix.heading = atof(field[1]);
-#endif /* HEADING_FIX */
+    session->gpsdata.fix.track = atof(field[1]);
     session->gpsdata.headingStatus = *field[2];
-#ifndef HEADING_FIX
-    mask |= (TRACK_SET | MODE_SET);
-    session->gpsdata.status = STATUS_FIX;	/* could be DGPS_FIX, we can't tell */
-    //session->gpsdata.newdata.mode = MODE_2D;
-    session->gpsdata.newdata.mode = MODE_2D;
-    //session->gpsdata.newdata.mode = MODE_NO_FIX;
-
-    session->gpsdata.newdata.pitch = atof(field[3]);
-#else /* HEADING_FIX */
-    // this device does not provide track, mode, or status - we have to rely on other devices for that!
-//    mask |= (TRACK_SET | MODE_SET);
-//    session->gpsdata.status = STATUS_FIX;	/* could be DGPS_FIX, we can't tell */
-//    session->gpsdata.newdata.mode = MODE_2D;
-    
-    session->gpsdata.fix.pitch = atof(field[3]);
-#endif /* HEADING_FIX */
+    session->gpsdata.fix.climb = atof(field[3]);
     session->gpsdata.pitchStatus = *field[4];
-
-#ifndef HEADING_FIX
-    session->gpsdata.newdata.roll = atof(field[5]);
-#else /* HEADING_FIX */
-    session->gpsdata.fix.roll = atof(field[5]);
-#endif /* HEADING_FIX */
+    session->gpsdata.fix.speed = atof(field[5]);
     session->gpsdata.rollStatus = *field[6];
-
-#ifndef HEADING_FIX
-    session->gpsdata.newdata.dip = atof(field[7]);
-#else /* HEADING_FIX */
-    session->gpsdata.fix.dip = atof(field[7]);
-#endif /* HEADING_FIX */
+    session->gpsdata.fix.altitude = atof(field[7]);
     session->gpsdata.horzField = atof(field[8]);
+    session->gpsdata.fix.mode = MODE_3D;
+    mask |= (STATUS_SET | MODE_SET | TRACK_SET | SPEED_SET | CLIMB_SET | ALTITUDE_SET);
+    session->gpsdata.status = STATUS_FIX;	/* could be DGPS_FIX */
 
-#ifndef HEADING_FIX
-    //gpsd_report(5, "Heading %lf  %c.\n", session->gpsdata.fix.track, session->gpsdata.fix.headingStatus);
-#else /* HEADING_FIX */
-    //gpsd_report(5, "Heading %lf  %c.\n", session->gpsdata.fix.heading, session->gpsdata.fix.headingStatus);
-#endif /* HEADING_FIX */
+    gpsd_report(LOG_RAW, "Heading %lf  %c.\n", session->gpsdata.fix.track, session->gpsdata.headingStatus);
     return mask;
 }
 #endif /* TNT_ENABLE */
@@ -574,7 +584,7 @@ static short nmea_checksum(char *sentence, unsigned char *correct_sum)
     if (correct_sum)
         *correct_sum = sum;
     (void)snprintf(csum, sizeof(csum), "%02X", sum);
-    return(toupper(csum[0])==toupper(p[0]))&&(toupper(csum[1])==toupper(p[1]));
+    return(csum[0]==toupper(p[0])) && (csum[1]==toupper(p[1]));
 }
 #endif /* __ UNUSED__ */
 
@@ -590,21 +600,22 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t *session)
     typedef gps_mask_t (*nmea_decoder)(int count, char *f[], struct gps_device_t *session);
     static struct {
 	char *name;
+	int nf;		/* minimum number of fields required to parse */
 	nmea_decoder decoder;
     } nmea_phrase[] = {
-	{"RMC", 	processGPRMC},
-	{"GGA",         processGPGGA},
-	{"GLL", 	processGPGLL},
-	{"GSA", 	processGPGSA},
-	{"GSV", 	processGPGSV},
-	{"VTG", 	NULL},		/* ignore Velocity Track made Good */
-	{"ZDA", 	processGPZDA},
-	{"PGRMC",	NULL},		/* ignore Garmin Sensor Config */
-	{"PGRME",	processPGRME},
-	{"PGRMI",	NULL},		/* ignore Garmin Sensor Init */
-	{"PGRMO",	NULL},		/* ignore Garmin Sentence Enable */
+	{"RMC", 8,	processGPRMC},
+	{"GGA", 13,	processGPGGA},
+	{"GLL", 7, 	processGPGLL},
+	{"GSA", 17,	processGPGSA},
+	{"GSV", 0,	processGPGSV},
+	{"VTG", 0, 	NULL},		/* ignore Velocity Track made Good */
+	{"ZDA", 7, 	processGPZDA},
+	{"PGRMC", 0,	NULL},		/* ignore Garmin Sensor Config */
+	{"PGRME", 7,	processPGRME},
+	{"PGRMI", 0,	NULL},		/* ignore Garmin Sensor Init */
+	{"PGRMO", 0,	NULL},		/* ignore Garmin Sentence Enable */
 #ifdef TNT_ENABLE
-	{"PTNTHTM",	processTNTHTM},
+	{"PTNTHTM", 9,	processTNTHTM},
 #endif /* TNT_ENABLE */
     };
     unsigned char buf[NMEA_MAX+1];
@@ -617,12 +628,25 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t *session)
     unsigned char sum;
 
     if (!nmea_checksum(sentence+1, &sum)) {
-        gpsd_report(1, "Bad NMEA checksum: '%s' should be %02X\n",
+        gpsd_report(LOG_ERROR, "Bad NMEA checksum: '%s' should be %02X\n",
                    sentence, sum);
         return 0;
     }
 #endif /* __ UNUSED__ */
 
+    /*
+     * We've had reports that on the Garmin GPS-10 the device sometimes 
+     * (1:1000 or so) sends garbage packets that have a valid checksum 
+     * but are like 2 successive NMEA packets merged together in one 
+     * with some fields lost.  Usually these are much longer than the
+     * legal limit for NMEA, so we can cope by just tossing out overlong
+     * packets.  This may be a generic bug of all Garmin chipsets.
+     */
+    if (strlen(sentence) > NMEA_MAX) {
+	gpsd_report(LOG_WARN, "Overlong packet rejected.\n");
+	return ONLINE_SET;
+    }
+
     /*@ -usedef @*//* splint 3.1.1 seems to have a bug here */
     /* make an editable copy of the sentence */
     strncpy((char *)buf, sentence, NMEA_MAX);
@@ -640,7 +664,7 @@ gps_mask_t nmea_parse(char *sentence, struct gps_device_t *session)
 	if (strlen(nmea_phrase[i].name) == 3)
 	    s += 2;	/* skip talker ID */
         if (strcmp(nmea_phrase[i].name, s) == 0) {
-	    if (nmea_phrase[i].decoder) {
+	    if (nmea_phrase[i].decoder!=NULL && (count >= nmea_phrase[i].nf)) {
 		retval = (nmea_phrase[i].decoder)(count, field, session);
 		strncpy(session->gpsdata.tag, nmea_phrase[i].name, MAXTAGLEN);
 		session->gpsdata.sentence_length = strlen(sentence);
@@ -663,7 +687,7 @@ void nmea_add_checksum(char *sentence)
     if (*p == '$') {
 	p++;
     } else {
-        gpsd_report(1, "Bad NMEA sentence: '%s'\n", sentence);
+        gpsd_report(LOG_ERROR, "Bad NMEA sentence: '%s'\n", sentence);
     }
     while ( ((c = *p) != '*') && (c != '\0')) {
 	sum ^= c;
@@ -684,16 +708,17 @@ int nmea_send(int fd, const char *fmt, ... )
     (void)vsnprintf(buf, sizeof(buf)-5, fmt, ap);
     va_end(ap);
     if (fmt[0] == '$') {
-	strcat(buf, "*");
+	(void)strlcat(buf, "*", BUFSIZ);
 	nmea_add_checksum(buf);
     } else
-	strcat(buf, "\r\n");
+	(void)strlcat(buf, "\r\n", BUFSIZ);
     status = (int)write(fd, buf, strlen(buf));
+    (void)tcdrain(fd);
     if (status == (int)strlen(buf)) {
-	gpsd_report(2, "=> GPS: %s\n", buf);
+	gpsd_report(LOG_IO, "=> GPS: %s\n", buf);
 	return status;
     } else {
-	gpsd_report(2, "=> GPS: %s FAILED\n", buf);
+	gpsd_report(LOG_WARN, "=> GPS: %s FAILED\n", buf);
 	return -1;
     }
 }
