@@ -1,3 +1,4 @@
+/* $Id: timebase.h 4040 2006-12-01 03:41:42Z esr $ */
 /* timebase.h -- constants that will require patching over time */
 
 /*
@@ -18,9 +19,9 @@
 #define LEAP_SECONDS	14
 
 /* IERS says "NO positive leap second will be introduced at the end of 
- * June 2006, so start subframe checking at the *next* 6-month boundary.
+ * December 2006, so start subframe checking at the *next* 6-month boundary.
  */
-#define START_SUBFRAME	1167627600	/* 1 Jan 2007 00:00:00 */
+#define START_SUBFRAME	1183262400	/* 1 Jun 2007 00:00:00 */
 
 /*
  * This is used only when an NMEA device issues a two-digit year in a GPRMC
