@@ -1,11 +1,14 @@
+/* $Id: tsip.c 4030 2006-11-30 07:29:25Z esr $ */
 /*
  * Handle the Trimble TSIP packet format
  * by Rob Janssen, PE1CHL.
  */
+#include <sys/types.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <math.h>
+#include "gpsd_config.h"
 #include "gpsd.h"
 #include "bits.h"
 
@@ -18,9 +21,10 @@
 
 static int tsip_write(int fd, unsigned int id, unsigned char *buf, size_t len)
 {
+#ifdef ALLOW_RECONFIGURE
     char buf2[BUFSIZ];
 
-    gpsd_report(5, "Sent TSIP packet id 0x%02x: %s\n",id,gpsd_hexdump(buf,len));
+    gpsd_report(LOG_IO, "Sent TSIP packet id 0x%02x: %s\n",id,gpsd_hexdump(buf,len));
 
     /*@ +charint @*/
     buf2[0] = '\x10';
@@ -43,42 +47,58 @@ static int tsip_write(int fd, unsigned int id, unsigned char *buf, size_t len)
 	return -1;
 
     return 0;
+#else
+    gpsd_report(LOG_IO, "Not sending TSIP packet id 0x%02x: %s\n",id,gpsd_hexdump(buf,len));
+    return -1;
+#endif /* ALLOW_RECONFIGURE */
 }
 
-static void tsip_initializer(struct gps_device_t *session)
+static void tsip_probe_subtype(struct gps_device_t *session, unsigned int seq)
 {
     unsigned char buf[100];
 
-    /* TSIP is ODD parity 1 stopbit, save original values and change it */
-    session->driver.tsip.parity = session->gpsdata.parity;
-    session->driver.tsip.stopbits = session->gpsdata.stopbits;
-    gpsd_set_speed(session, session->gpsdata.baudrate, 'O', 1);
+    switch (seq) {
+    case 0:
+	/* TSIP is ODD parity 1 stopbit, save original values and change it */
+	/* XXX this is not always a good idea; not all receivers are 8O1... */
+	session->driver.tsip.parity = session->gpsdata.parity;
+	session->driver.tsip.stopbits = session->gpsdata.stopbits;
+	gpsd_set_speed(session, session->gpsdata.baudrate, 'O', 1);
+	break;
 
-    /* I/O Options */
-    putbyte(buf,0,0x1e);		/* Position: DP, MSL, LLA */
-    putbyte(buf,1,0x02);		/* Velocity: ENU */
-    putbyte(buf,2,0x00);		/* Time: GPS */
-    putbyte(buf,3,0x08);		/* Aux: dBHz */
-    (void)tsip_write(session->gpsdata.gps_fd, 0x35, buf, 4);
-
-    /* Request Software Versions */
-    (void)tsip_write(session->gpsdata.gps_fd, 0x1f, buf, 0);
-
-    /* Request Current Time */
-    (void)tsip_write(session->gpsdata.gps_fd, 0x21, buf, 0);
-
-    /* Request GPS Systems Message */
-    (void)tsip_write(session->gpsdata.gps_fd, 0x28, buf, 0);
-
-    /* Request Current Datum Values */
-    putbyte(buf,0,0x15);
-    (void)tsip_write(session->gpsdata.gps_fd, 0x8e, buf, 1);
-
-    /* Request Navigation Configuration */
-    putbyte(buf,0,0x03);
-    (void)tsip_write(session->gpsdata.gps_fd, 0xbb, buf, 1);
+    case 1:
+	/* Request Software Versions */
+	(void)tsip_write(session->gpsdata.gps_fd, 0x1f, NULL, 0);
+	/* Request Current Time */
+	(void)tsip_write(session->gpsdata.gps_fd, 0x21, NULL, 0);
+	/* Request GPS Systems Message */
+	(void)tsip_write(session->gpsdata.gps_fd, 0x28, NULL, 0);
+	/* Request Current Datum Values */
+	putbyte(buf,0,0x15);
+	(void)tsip_write(session->gpsdata.gps_fd, 0x8e, buf, 1);
+	/* Request Navigation Configuration */
+	putbyte(buf,0,0x03);
+	(void)tsip_write(session->gpsdata.gps_fd, 0xbb, buf, 1);
+	break;
+    }
 }
 
+#ifdef ALLOW_RECONFIGURE
+static void tsip_configurator(struct gps_device_t *session, unsigned int seq)
+{
+    if (seq == 0) {
+	unsigned char buf[100];
+
+	/* I/O Options */
+	putbyte(buf,0,0x1e);		/* Position: DP, MSL, LLA */
+	putbyte(buf,1,0x02);		/* Velocity: ENU */
+	putbyte(buf,2,0x00);		/* Time: GPS */
+	putbyte(buf,3,0x08);		/* Aux: dBHz */
+	(void)tsip_write(session->gpsdata.gps_fd, 0x35, buf, 4);
+    }
+}
+#endif /* ALLOW_RECONFIGURE */
+
 static void tsip_wrapup(struct gps_device_t *session)
 {
     /* restore saved parity and stopbits when leaving TSIP mode */
@@ -124,44 +144,44 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
     unsigned char buf[BUFSIZ];
     char buf2[BUFSIZ];
 
-    if (session->packet_type != TSIP_PACKET) {
-	gpsd_report(2, "tsip_analyze packet type %d\n",session->packet_type);
+    if (session->packet.type != TSIP_PACKET) {
+	gpsd_report(LOG_INF, "tsip_analyze packet type %d\n",session->packet.type);
 	return 0;
     }
 
     /*@ +charint @*/
-    if (session->outbuflen < 4 || session->outbuffer[0] != 0x10)
+    if (session->packet.outbuflen < 4 || session->packet.outbuffer[0] != 0x10)
 	return 0;
 
     /* remove DLE stuffing and put data part of message in buf */
 
     memset(buf, 0, sizeof(buf));
     buf2[len = 0] = '\0';
-    for (i = 2; i < (int)session->outbuflen; i++) {
-	if (session->outbuffer[i] == 0x10)
-	    if (session->outbuffer[++i] == 0x03)
+    for (i = 2; i < (int)session->packet.outbuflen; i++) {
+	if (session->packet.outbuffer[i] == 0x10)
+	    if (session->packet.outbuffer[++i] == 0x03)
 		break;
 
 	(void)snprintf(buf2+strlen(buf2), 
 		      sizeof(buf2)-strlen(buf2),
-		      "%02x", buf[len++] = session->outbuffer[i]);
+		      "%02x", buf[len++] = session->packet.outbuffer[i]);
     }
     /*@ -charint @*/
 
     (void)snprintf(session->gpsdata.tag, sizeof(session->gpsdata.tag), 
-		   "ID%02x", id = (unsigned)session->outbuffer[1]);
+		   "ID%02x", id = (unsigned)session->packet.outbuffer[1]);
 
-    gpsd_report(5, "TSIP packet id 0x%02x length %d: %s\n",id,len,buf2);
+    gpsd_report(LOG_IO, "TSIP packet id 0x%02x length %d: %s\n",id,len,buf2);
     (void)time(&now);
 
     switch (id) {
     case 0x13:		/* Packet Received */
 	u1 = getub(buf,0);
 	u2 = getub(buf,1);
-	gpsd_report(4, "Received packet of type %02x cannot be parsed\n",u1);
+	gpsd_report(LOG_WARN, "Received packet of type %02x cannot be parsed\n",u1);
 #if USE_SUPERPACKET
 	if ((int)u1 == 0x8e && (int)u2 == 0x23) { /* no Compact Super Packet */
-	    gpsd_report(2, "No Compact Super Packet, use LFwEI\n");
+	    gpsd_report(LOG_WARN, "No Compact Super Packet, use LFwEI\n");
 
 	    /* Request LFwEI Super Packet */
 	    putbyte(buf,0,0x20);
@@ -185,11 +205,12 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    session->gpsdata.sentence_time = gpstime_to_unix((int)s1, f1) - f2;
 
 #ifdef NTPSHM_ENABLE
-	    (void)ntpshm_put(session, session->gpsdata.sentence_time + 0.075);
+	    if (session->context->enable_ntpshm)
+		(void)ntpshm_put(session,session->gpsdata.sentence_time+0.075);
 #endif
 	    mask |= TIME_SET;
 	}
-	gpsd_report(4, "GPS Time %f %d %f\n",f1,s1,f2);
+	gpsd_report(LOG_INF, "GPS Time %f %d %f\n",f1,s1,f2);
 	break;
     case 0x42:		/* Single-Precision Position Fix, XYZ ECEF */
 	if (len != 16)
@@ -198,7 +219,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f2 = getf(buf,4);			/* Y */
 	f3 = getf(buf,8);			/* Z */
 	f4 = getf(buf,12);			/* time-of-fix */
-	gpsd_report(4, "GPS Position XYZ %f %f %f %f\n",f1,f2,f3,f4);
+	gpsd_report(LOG_INF, "GPS Position XYZ %f %f %f %f\n",f1,f2,f3,f4);
 	break;
     case 0x43:		/* Velocity Fix, XYZ ECEF */
 	if (len != 20)
@@ -208,14 +229,19 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f3 = getf(buf,8);			/* Z velocity */
 	f4 = getf(buf,12);			/* bias rate */
 	f5 = getf(buf,16);			/* time-of-fix */
-	gpsd_report(4, "GPS Velocity XYZ %f %f %f %f %f\n",f1,f2,f3,f4,f5);
+	gpsd_report(LOG_INF, "GPS Velocity XYZ %f %f %f %f %f\n",f1,f2,f3,f4,f5);
 	break;
     case 0x45:		/* Software Version Information */
 	if (len != 10)
 	    break;
-	gpsd_report(4, "Software versions %d.%d %02d%02d%02d %d.%d %02d%02d%02d\n",
-		getub(buf,0),getub(buf,1),getub(buf,4),getub(buf,2),getub(buf,3),
-		getub(buf,5),getub(buf,6),getub(buf,9),getub(buf,7),getub(buf,8));
+	/*@ -formattype @*/
+	(void)snprintf(session->subtype, sizeof(session->subtype), 
+		       "%d.%d %02d%02d%02d %d.%d %02d%02d%02d",
+		       getub(buf,0),getub(buf,1),getub(buf,4),getub(buf,2),getub(buf,3),
+		       getub(buf,5),getub(buf,6),getub(buf,9),getub(buf,7),getub(buf,8));
+	/*@ +formattype @*/
+	gpsd_report(LOG_INF, "Software version: %s\n", session->subtype);
+	mask |= DEVICEID_SET;
 	break;
     case 0x46:		/* Health of Receiver */
 	if (len != 2)
@@ -233,7 +259,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
                 mask |= STATUS_SET;
             }
         }
-	gpsd_report(4, "Receiver health %02x %02x\n",u1,u2);
+	gpsd_report(LOG_PROG, "Receiver health %02x %02x\n",u1,u2);
 	break;
     case 0x47:		/* Signal Levels for all Satellites */
 	count = (int)getub(buf,0);		/* satellite count */
@@ -252,27 +278,27 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    (void)snprintf(buf2+strlen(buf2), sizeof(buf2)-strlen(buf2),
 			   " %d=%.1f",(int)u1,f1);
 	}
-	gpsd_report(4, "Signal Levels (%d):%s\n",count,buf2);
+	gpsd_report(LOG_PROG, "Signal Levels (%d):%s\n",count,buf2);
 	mask |= SATELLITE_SET;
 	break;
     case 0x48:		/* GPS System Message */
 	buf[len] = '\0';
-	gpsd_report(4, "GPS System Message: %s\n",buf);
+	gpsd_report(LOG_PROG, "GPS System Message: %s\n",buf);
 	break;
     case 0x4a:		/* Single-Precision Position LLA */
 	if (len != 20)
 	    break;
-	session->gpsdata.newdata.latitude  = getf(buf,0) * RAD_2_DEG;
-	session->gpsdata.newdata.longitude = getf(buf,4) * RAD_2_DEG;
-	session->gpsdata.newdata.altitude  = getf(buf,8);
+	session->gpsdata.fix.latitude  = getf(buf,0) * RAD_2_DEG;
+	session->gpsdata.fix.longitude = getf(buf,4) * RAD_2_DEG;
+	session->gpsdata.fix.altitude  = getf(buf,8);
 	f1 = getf(buf,12);			/* clock bias */
 	f2 = getf(buf,16);			/* time-of-fix */
 	if (session->driver.tsip.gps_week) {
-	    session->gpsdata.newdata.time = session->gpsdata.sentence_time =
+	    session->gpsdata.fix.time = session->gpsdata.sentence_time =
 		gpstime_to_unix((int)session->driver.tsip.gps_week, f2) - session->context->leap_seconds;
 	    mask |= TIME_SET;
 	}
-	gpsd_report(4, "GPS LLA %f %f %f\n",session->gpsdata.newdata.latitude,session->gpsdata.newdata.longitude,session->gpsdata.newdata.altitude);
+	gpsd_report(LOG_PROG, "GPS LLA %f %f %f\n",session->gpsdata.fix.latitude,session->gpsdata.fix.longitude,session->gpsdata.fix.altitude);
 	mask |= LATLON_SET | ALTITUDE_SET | CYCLE_START_SET;
 	break;
     case 0x4b:		/* Machine/Code ID and Additional Status */
@@ -281,10 +307,10 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	u1 = getub(buf,0);			/* Machine ID */
 	u2 = getub(buf,1);			/* Status 1 */
 	u3 = getub(buf,2);			/* Status 2 */
-	gpsd_report(4, "Machine ID %02x %02x %02x\n",u1,u2,u3);
+	gpsd_report(LOG_INF, "Machine ID %02x %02x %02x\n",u1,u2,u3);
 #if USE_SUPERPACKET
 	if ((u3 & 0x01) != (u_int8_t)0 && !session->driver.tsip.superpkt) {
-	    gpsd_report(2, "Switching to Super Packet mode\n");
+	    gpsd_report(LOG_PROG, "Switching to Super Packet mode\n");
 
 	    /* set new I/O Options for Super Packet output */
 	    putbyte(buf,0,0x2c);		/* Position: SP, MSL */
@@ -303,7 +329,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	u2 = getub(buf,1);			/* Velocity */
 	u3 = getub(buf,2);			/* Timing */
 	u4 = getub(buf,3);			/* Aux */
-	gpsd_report(4, "IO Options %02x %02x %02x %02x\n",u1,u2,u3,u4);
+	gpsd_report(LOG_INF, "IO Options %02x %02x %02x %02x\n",u1,u2,u3,u4);
 #if USE_SUPERPACKET
 	if ((u1 & 0x20) != (u_int8_t)0) {	/* Output Super Packets? */
 	    /* No LFwEI Super Packet */
@@ -326,13 +352,13 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f3 = getf(buf,8);			/* Up velocity */
 	f4 = getf(buf,12);			/* clock bias rate */
 	f5 = getf(buf,16);			/* time-of-fix */
-	session->gpsdata.newdata.climb = f3;
+	session->gpsdata.fix.climb = f3;
 	/*@ -evalorder @*/
-	session->gpsdata.newdata.speed = sqrt(pow(f2,2) + pow(f1,2)) * MPS_TO_KNOTS;
+	session->gpsdata.fix.speed = sqrt(pow(f2,2) + pow(f1,2));
 	/*@ +evalorder @*/
-	if ((session->gpsdata.newdata.track = atan2(f1,f2) * RAD_2_DEG) < 0)
-	    session->gpsdata.newdata.track += 360.0;
-	gpsd_report(4, "GPS Velocity ENU %f %f %f %f %f\n",f1,f2,f3,f4,f5);
+	if ((session->gpsdata.fix.track = atan2(f1,f2) * RAD_2_DEG) < 0)
+	    session->gpsdata.fix.track += 360.0;
+	gpsd_report(LOG_INF, "GPS Velocity ENU %f %f %f %f %f\n",f1,f2,f3,f4,f5);
 	mask |= SPEED_SET | TRACK_SET | CLIMB_SET; 
 	break;
     case 0x57:		/* Information About Last Computed Fix */
@@ -346,7 +372,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	if (getub(buf,0) == 0x01)		/* good current fix? */
 	    session->driver.tsip.gps_week = s1;
 	/*@ -charint @*/
-	gpsd_report(4, "Fix info %02x %02x %d %f\n",u1,u2,s1,f1);
+	gpsd_report(LOG_INF, "Fix info %02x %02x %d %f\n",u1,u2,s1,f1);
 	break;
     case 0x58:		/* Satellite System Data/Acknowledge from Receiver */
 	break;
@@ -359,7 +385,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f2 = getf(buf,9);			/* Code phase */
 	f3 = getf(buf,13);			/* Doppler */
 	d1 = getd(buf,17);			/* Time of Measurement */
-	gpsd_report(4, "Raw Measurement Data %d %f %f %f %f\n",getub(buf,0),f1,f2,f3,d1);
+	gpsd_report(LOG_PROG, "Raw Measurement Data %d %f %f %f %f\n",getub(buf,0),f1,f2,f3,d1);
 	break;
     case 0x5c:		/* Satellite Tracking Status */
 	if (len != 24)
@@ -374,7 +400,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	d1 = getf(buf,12) * RAD_2_DEG;		/* Elevation */
 	d2 = getf(buf,16) * RAD_2_DEG;		/* Azimuth */
 	i = (int)(u2 >> 3);			/* channel number */
-	gpsd_report(4, "Satellite Tracking Status %d: %2d 0x%02x %d %d %.1f %f %.1f %.1f\n",i,u1,u2,u3,u4,f1,f2,d1,d2);
+	gpsd_report(LOG_INF, "Satellite Tracking Status: Ch %2d PRN %3d Res %d Acq %d Eph %2d SNR %4.1f LMT %.04f El %4.1f Az %5.1f\n",i,u1,u2&7,u3,u4,f1,f2,d1,d2);
 	if (i < TSIP_CHANNELS) {
 	    if (d1 >= 0.0) {
 		session->gpsdata.PRN[i] = (int)u1;
@@ -401,15 +427,15 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	{
 	case 3:
 	    //session->gpsdata.status = STATUS_FIX;
-	    session->gpsdata.newdata.mode = MODE_2D;
+	    session->gpsdata.fix.mode = MODE_2D;
 	    break;
 	case 4:
 	    //session->gpsdata.status = STATUS_FIX;
-	    session->gpsdata.newdata.mode = MODE_3D;
+	    session->gpsdata.fix.mode = MODE_3D;
 	    break;
 	default:
 	    //session->gpsdata.status = STATUS_NO_FIX;
-	    session->gpsdata.newdata.mode = MODE_NO_FIX;
+	    session->gpsdata.fix.mode = MODE_NO_FIX;
 	    break;
 	}
 	session->gpsdata.satellites_used = count;
@@ -429,11 +455,13 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 			" %d",session->gpsdata.used[i] = getub(buf,17 + i));
 	/*@ -charint @*/
 
-	gpsd_report(4, "Sat info: %d %f %f %f %f %f %d:%s\n",
-		session->gpsdata.newdata.mode, session->gpsdata.pdop,
-		session->gpsdata.hdop, session->gpsdata.vdop,
-		session->gpsdata.tdop, session->gpsdata.gdop,
-		session->gpsdata.satellites_used,buf2);
+	gpsd_report(LOG_INF, "Sat info: mode %d, satellites used %d: %s\n",
+	    session->gpsdata.fix.mode, session->gpsdata.satellites_used,buf2);
+	gpsd_report(LOG_INF,
+	    "Sat info: DOP P=%.1f H=%.1f V=%.1f T=%.1f G=%.1f\n",
+	    session->gpsdata.pdop, session->gpsdata.hdop,
+	    session->gpsdata.vdop, session->gpsdata.tdop,
+	    session->gpsdata.gdop);
         mask |= HDOP_SET | VDOP_SET | PDOP_SET | TDOP_SET | GDOP_SET | STATUS_SET | MODE_SET | USED_SET;
 	break;
     case 0x6e:		/* Synchronized Measurements */
@@ -460,7 +488,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    mask |= STATUS_SET;
 	}
 	/*@ -charint @*/
-	gpsd_report(4, "DGPS mode %d\n",u1);
+	gpsd_report(LOG_INF, "DGPS mode %d\n",u1);
 	break;
     case 0x83:		/* Double-Precision XYZ Position Fix and Bias Information */
 	if (len != 36)
@@ -470,22 +498,22 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	d3 = getd(buf,16);			/* Z */
 	d4 = getd(buf,24);			/* clock bias */
 	f1 = getf(buf,32);			/* time-of-fix */
-	gpsd_report(4, "GPS Position XYZ %f %f %f %f %f\n",d1,d2,d3,d4,f1);
+	gpsd_report(LOG_INF, "GPS Position XYZ %f %f %f %f %f\n",d1,d2,d3,d4,f1);
 	break;
     case 0x84:		/* Double-Precision LLA Position Fix and Bias Information */
 	if (len != 36)
 	    break;
-	session->gpsdata.newdata.latitude  = getd(buf,0) * RAD_2_DEG;
-	session->gpsdata.newdata.longitude = getd(buf,8) * RAD_2_DEG;
-	session->gpsdata.newdata.altitude  = getd(buf,16);
+	session->gpsdata.fix.latitude  = getd(buf,0) * RAD_2_DEG;
+	session->gpsdata.fix.longitude = getd(buf,8) * RAD_2_DEG;
+	session->gpsdata.fix.altitude  = getd(buf,16);
 	d1 = getd(buf,24);			/* clock bias */
 	f1 = getf(buf,32);			/* time-of-fix */
 	if (session->driver.tsip.gps_week) {
-	    session->gpsdata.newdata.time = session->gpsdata.sentence_time =
+	    session->gpsdata.fix.time = session->gpsdata.sentence_time =
 		gpstime_to_unix((int)session->driver.tsip.gps_week, f1) - session->context->leap_seconds;
 	    mask |= TIME_SET;
 	}
-	gpsd_report(4, "GPS DP LLA %f %f %f\n",session->gpsdata.newdata.latitude,session->gpsdata.newdata.longitude,session->gpsdata.newdata.altitude);
+	gpsd_report(LOG_INF, "GPS DP LLA %f %f %f\n",session->gpsdata.fix.latitude,session->gpsdata.fix.longitude,session->gpsdata.fix.altitude);
 	mask |= LATLON_SET | ALTITUDE_SET | CYCLE_START_SET;
 	break;
     case 0x8f:		/* Super Packet.  Well...  */
@@ -505,11 +533,12 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    d3 = getd(buf,19);			/* DZ */
 	    d4 = getd(buf,27);			/* A-axis */
 	    d5 = getd(buf,35);			/* Eccentricity Squared */
-	    gpsd_report(4, "Current Datum %d %f %f %f %f %f\n",s1,d1,d2,d3,d4,d5);
+	    gpsd_report(LOG_INF, "Current Datum %d %f %f %f %f %f\n",s1,d1,d2,d3,d4,d5);
 	    break;
 
 	case 0x20:	/* Last Fix with Extra Information (binary fixed point) */
-	    if (len != 56)
+	    /* XXX CSK sez "why does my Lassen iQ output oversize packets?" */
+	    if ((len != 56) && (len != 64))
 		break;
 	    s1 = getsw(buf,2);			/* east velocity */
 	    s2 = getsw(buf,4);			/* north velocity */
@@ -524,7 +553,7 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    u4 = getub(buf,29);			/* utc offset */
 	    s4 = getsw(buf,30);			/* tsip.gps_week */
 						/* PRN/IODE data follows */
-	    gpsd_report(4, "LFwEI %d %d %d %u %d %u %u %x %x %u %u %d\n",s1,s2,s3,ul1,sl1,ul2,sl2,u1,u2,u3,u4,s4);
+	    gpsd_report(LOG_RAW, "LFwEI %d %d %d %u %d %u %u %x %x %u %u %d\n",s1,s2,s3,ul1,sl1,ul2,sl2,u1,u2,u3,u4,s4);
 
 	    if ((u1 & 0x01) != (u_int8_t)0)	/* check velocity scaling */
 		d5 = 0.02;
@@ -532,27 +561,27 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		d5 = 0.005;
 	    d1 = s1 * d5;			/* east velocity m/s */
 	    d2 = s2 * d5;			/* north velocity m/s */
-	    session->gpsdata.newdata.climb = s3 * d5; /* up velocity m/s */
+	    session->gpsdata.fix.climb = s3 * d5; /* up velocity m/s */
 	    /*@ -evalorder @*/
-	    session->gpsdata.newdata.speed = sqrt(pow(d2,2) + pow(d1,2)) * MPS_TO_KNOTS;
+	    session->gpsdata.fix.speed = sqrt(pow(d2,2) + pow(d1,2));
 	    /*@ +evalorder @*/
-	    if ((session->gpsdata.newdata.track = atan2(d1,d2) * RAD_2_DEG) < 0)
-		session->gpsdata.newdata.track += 360.0;
-	    session->gpsdata.newdata.latitude  = sl1 * SEMI_2_DEG;
-	    if ((session->gpsdata.newdata.longitude = ul2 * SEMI_2_DEG) > 180.0)
-		session->gpsdata.newdata.longitude -= 360.0;
-	    session->gpsdata.separation = wgs84_separation(session->gpsdata.newdata.latitude, session->gpsdata.newdata.longitude);
-	    session->gpsdata.newdata.altitude  = sl2 * 1e-3 - session->gpsdata.separation;;
+	    if ((session->gpsdata.fix.track = atan2(d1,d2) * RAD_2_DEG) < 0)
+		session->gpsdata.fix.track += 360.0;
+	    session->gpsdata.fix.latitude  = sl1 * SEMI_2_DEG;
+	    if ((session->gpsdata.fix.longitude = ul2 * SEMI_2_DEG) > 180.0)
+		session->gpsdata.fix.longitude -= 360.0;
+	    session->gpsdata.separation = wgs84_separation(session->gpsdata.fix.latitude, session->gpsdata.fix.longitude);
+	    session->gpsdata.fix.altitude  = sl2 * 1e-3 - session->gpsdata.separation;;
 	    session->gpsdata.status = STATUS_NO_FIX;
-	    session->gpsdata.newdata.mode = MODE_NO_FIX;
+	    session->gpsdata.fix.mode = MODE_NO_FIX;
 	    if ((u2 & 0x01) == (u_int8_t)0) {	/* Fix Available */
 		session->gpsdata.status = STATUS_FIX;
 		if ((u2 & 0x02) != (u_int8_t)0)	/* DGPS Corrected */
 		    session->gpsdata.status = STATUS_DGPS_FIX;
 		if ((u2 & 0x04) != (u_int8_t)0)	/* Fix Dimension */
-		    session->gpsdata.newdata.mode = MODE_2D;
+		    session->gpsdata.fix.mode = MODE_2D;
 		else
-		    session->gpsdata.newdata.mode = MODE_3D;
+		    session->gpsdata.fix.mode = MODE_3D;
 	    }
 	    session->gpsdata.satellites_used = (int)u3;
 	    if ((int)u4 > 10) {
@@ -560,12 +589,13 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 		session->context->valid |= LEAP_SECOND_VALID;
 	    }
 	    session->driver.tsip.gps_week = s4;
-	    session->gpsdata.newdata.time = session->gpsdata.sentence_time =
+	    session->gpsdata.fix.time = session->gpsdata.sentence_time =
 		gpstime_to_unix((int)s4, ul1 * 1e-3) - session->context->leap_seconds;
 	    mask |= TIME_SET | LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET | STATUS_SET | MODE_SET | CYCLE_START_SET; 
 	    break;
 	case 0x23:	/* Compact Super Packet */
-	    if (len != 29)
+	    /* XXX CSK sez "i don't trust this to not be oversized either." */
+	    if (len < 29)
 		break;
 	    ul1 = getul(buf,1);			/* time */
 	    s1 = getsw(buf,5);			/* tsip.gps_week */
@@ -577,46 +607,46 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	    s2 = getsw(buf,21);			/* east velocity */
 	    s3 = getsw(buf,23);			/* north velocity */
 	    s4 = getsw(buf,25);			/* up velocity */
-	    gpsd_report(4, "CSP %u %d %u %u %d %u %d %d %d %d\n",ul1,s1,u1,u2,sl1,ul2,sl3,s2,s3,s4);
+	    gpsd_report(LOG_INF, "CSP %u %d %u %u %d %u %d %d %d %d\n",ul1,s1,u1,u2,sl1,ul2,sl3,s2,s3,s4);
 	    session->driver.tsip.gps_week = s1;
 	    if ((int)u1 > 10) {
 		session->context->leap_seconds = (int)u1;
 		session->context->valid |= LEAP_SECOND_VALID;
 	    }
-	    session->gpsdata.newdata.time = session->gpsdata.sentence_time =
+	    session->gpsdata.fix.time = session->gpsdata.sentence_time =
 		gpstime_to_unix((int)s1, ul1 * 1e-3) - session->context->leap_seconds;
 	    session->gpsdata.status = STATUS_NO_FIX;
-	    session->gpsdata.newdata.mode = MODE_NO_FIX;
+	    session->gpsdata.fix.mode = MODE_NO_FIX;
 	    if ((u2 & 0x01) == (u_int8_t)0) {	/* Fix Available */
 		session->gpsdata.status = STATUS_FIX;
 		if ((u2 & 0x02) != (u_int8_t)0)	/* DGPS Corrected */
 		    session->gpsdata.status = STATUS_DGPS_FIX;
 		if ((u2 & 0x04) != (u_int8_t)0)	/* Fix Dimension */
-		    session->gpsdata.newdata.mode = MODE_2D;
+		    session->gpsdata.fix.mode = MODE_2D;
 		else
-		    session->gpsdata.newdata.mode = MODE_3D;
+		    session->gpsdata.fix.mode = MODE_3D;
 	    }
-	    session->gpsdata.newdata.latitude  = sl1 * SEMI_2_DEG;
-	    if ((session->gpsdata.newdata.longitude = ul2 * SEMI_2_DEG) > 180.0)
-		session->gpsdata.newdata.longitude -= 360.0;
-	    session->gpsdata.separation = wgs84_separation(session->gpsdata.newdata.latitude, session->gpsdata.newdata.longitude);
-	    session->gpsdata.newdata.altitude  = sl3 * 1e-3 - session->gpsdata.separation;;
+	    session->gpsdata.fix.latitude  = sl1 * SEMI_2_DEG;
+	    if ((session->gpsdata.fix.longitude = ul2 * SEMI_2_DEG) > 180.0)
+		session->gpsdata.fix.longitude -= 360.0;
+	    session->gpsdata.separation = wgs84_separation(session->gpsdata.fix.latitude, session->gpsdata.fix.longitude);
+	    session->gpsdata.fix.altitude  = sl3 * 1e-3 - session->gpsdata.separation;;
 	    if ((u2 & 0x20) != (u_int8_t)0)	/* check velocity scaling */
 		d5 = 0.02;
 	    else
 		d5 = 0.005;
 	    d1 = s2 * d5;			/* east velocity m/s */
 	    d2 = s3 * d5;			/* north velocity m/s */
-	    session->gpsdata.newdata.climb = s4 * d5; /* up velocity m/s */
+	    session->gpsdata.fix.climb = s4 * d5; /* up velocity m/s */
 	    /*@ -evalorder @*/
-	    session->gpsdata.newdata.speed = sqrt(pow(d2,2) + pow(d1,2)) * MPS_TO_KNOTS;
+	    session->gpsdata.fix.speed = sqrt(pow(d2,2) + pow(d1,2)) * MPS_TO_KNOTS;
 	    /*@ +evalorder @*/
-	    if ((session->gpsdata.newdata.track = atan2(d1,d2) * RAD_2_DEG) < 0)
-		session->gpsdata.newdata.track += 360.0;
+	    if ((session->gpsdata.fix.track = atan2(d1,d2) * RAD_2_DEG) < 0)
+		session->gpsdata.fix.track += 360.0;
 	    mask |= TIME_SET | LATLON_SET | ALTITUDE_SET | SPEED_SET | TRACK_SET | CLIMB_SET | STATUS_SET | MODE_SET | CYCLE_START_SET; 
 	    break;
 	default:
-	    gpsd_report(4,"Unhandled TSIP superpacket type 0x%02x\n",u1);
+	    gpsd_report(LOG_WARN,"Unhandled TSIP superpacket type 0x%02x\n",u1);
 	}
 	break;
     case 0xbb:		/* Navigation Configuration */
@@ -631,10 +661,10 @@ static gps_mask_t tsip_analyze(struct gps_device_t *session)
 	f3 = getf(buf,13);			/* DOP Mask */
 	f4 = getf(buf,17);			/* DOP Switch */
 	u5 = getub(buf,21);			/* DGPS Age Limit */
-	gpsd_report(4, "Navigation Configuration %u %u %u %u %f %f %f %f %u\n",u1,u2,u3,u4,f1,f2,f3,f4,u5);
+	gpsd_report(LOG_INF, "Navigation Configuration %u %u %u %u %f %f %f %f %u\n",u1,u2,u3,u4,f1,f2,f3,f4,u5);
 	break;
     default:
-	gpsd_report(4,"Unhandled TSIP packet type 0x%02x\n",id);
+	gpsd_report(LOG_WARN,"Unhandled TSIP packet type 0x%02x\n",id);
 	break;
     }
 
@@ -673,15 +703,15 @@ static gps_mask_t tsip_parse_input(struct gps_device_t *session)
 {
     gps_mask_t st;
 
-    if (session->packet_type == TSIP_PACKET){
+    if (session->packet.type == TSIP_PACKET){
 	st = tsip_analyze(session);
-	session->gpsdata.driver_mode = 1;
+	session->gpsdata.driver_mode = 1;	/* binary */
 	return st;
 #ifdef EVERMORE_ENABLE
-    } else if (session->packet_type == EVERMORE_PACKET) {
+    } else if (session->packet.type == EVERMORE_PACKET) {
 	(void)gpsd_switch_driver(session, "EverMore binary");
-	st = evermore_parse(session, session->outbuffer, session->outbuflen);
-	session->gpsdata.driver_mode = 0;
+	st = evermore_parse(session, session->packet.outbuffer, session->packet.outbuflen);
+	session->gpsdata.driver_mode = 1;	/* binary */
 	return st;
 #endif /* EVERMORE_ENABLE */
     } else
@@ -694,15 +724,22 @@ struct gps_type_t tsip_binary =
     .typename       = "Trimble TSIP",	/* full name of type */
     .trigger        = NULL,		/* no trigger */
     .channels       = TSIP_CHANNELS,	/* consumer-grade GPS */
-    .probe          = NULL,		/* no probe */
-    .initializer    = tsip_initializer,	/* initialization */
-    .get_packet     = packet_get,	/* use the generic packet getter */
+    .probe_wakeup   = NULL,		/* no wakeup to be done before hunt */
+    .probe_detect   = NULL,		/* no probe */
+    .probe_subtype  = tsip_probe_subtype,	/* no more subtype discovery */
+#ifdef ALLOW_RECONFIGURE
+    .configurator   = tsip_configurator,/* initial mode sets */
+#endif /* ALLOW_RECONFIGURE */
+    .get_packet     = generic_get,	/* use the generic packet getter */
     .parse_packet   = tsip_parse_input,	/* parse message packets */
     .rtcm_writer    = NULL,		/* doesn't accept DGPS corrections */
     .speed_switcher = tsip_speed_switch,/* change baud rate */
     .mode_switcher  = NULL,		/* no mode switcher */
     .rate_switcher  = NULL,		/* no rate switcher */
     .cycle_chars    = -1,		/* not relevant, no rate switcher */
+#ifdef ALLOW_RECONFIGURE
+    .revert         = NULL,		/* FIXME: revert sentence mix */
+#endif /* ALLOW_RECONFIGURE */
     .wrapup         = tsip_wrapup,	/* restore comms parameters */
     .cycle          = 1,		/* updates every second */
 };
