@@ -1,3 +1,5 @@
+/* $Id: xgps.c 3871 2006-11-13 00:40:00Z esr $ */
+#include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -5,6 +7,7 @@
 #include <unistd.h>
 #include <math.h>
 #include <errno.h>
+#include <stdbool.h>
 #include <Xm/Xm.h>
 #include <Xm/MwmUtil.h>
 #include <Xm/PushB.h>
@@ -17,7 +20,7 @@
 #include <Xm/Protocols.h>
 #include <X11/Shell.h>
 
-#include "config.h"
+#include "gpsd_config.h"
 #include "gps.h"
 #include "display.h"
 
@@ -37,11 +40,11 @@ static Widget quitbutton;
 static Widget satellite_list, satellite_diagram, status;
 static Widget rowColumn_11, rowColumn_12, rowColumn_13, rowColumn_14;
 static Widget rowColumn_15, rowColumn_16, rowColumn_17, rowColumn_18;
-static Widget rowColumn_19, rowColumn_20;
+static Widget rowColumn_19, rowColumn_20, rowColumn_21;
 static Widget text_1, text_2, text_3, text_4, text_5;
-static Widget text_6, text_7, text_8, text_9, text_10;
+static Widget text_6, text_7, text_8, text_9, text_10, text_11;
 static Widget label_1, label_2, label_3, label_4, label_5;
-static Widget label_6, label_7, label_8, label_9, label_10;
+static Widget label_6, label_7, label_8, label_9, label_10, label_11;
 static GC gc;
 
 /*@ -nullassign @*/
@@ -57,7 +60,7 @@ struct unit_t {
     double factor;
 };
 static struct unit_t speedtable[] = {
-    {"knots",		1/KNOTS_TO_KPH},
+    {"knots",		MPS_TO_KNOTS},
     {"mph",		MPS_TO_MPH},
     {"kph",		MPS_TO_KPH},
 }, *speedunits = speedtable;
@@ -175,17 +178,19 @@ static void build_gui(Widget toplevel)
     rowColumn_18 = XtCreateManagedWidget("epv      ", xmRowColumnWidgetClass, left, args, 1);
     rowColumn_19 = XtCreateManagedWidget("climb    ", xmRowColumnWidgetClass, left, args, 1);
     rowColumn_20 = XtCreateManagedWidget("status   ", xmRowColumnWidgetClass, right, args, 1);
+    rowColumn_21 = XtCreateManagedWidget("type     ", xmRowColumnWidgetClass, right, args, 1);
 
     label_1 = XtCreateManagedWidget("Time     ", xmLabelWidgetClass, rowColumn_11, args, 0);
     label_2 = XtCreateManagedWidget("Latitude ", xmLabelWidgetClass, rowColumn_12, args, 0);
     label_3 = XtCreateManagedWidget("Longitude", xmLabelWidgetClass, rowColumn_13, args, 0);
     label_4 = XtCreateManagedWidget("Altitude ", xmLabelWidgetClass, rowColumn_14, args, 0);
     label_5 = XtCreateManagedWidget("Speed    ", xmLabelWidgetClass, rowColumn_15, args, 0);
-    label_6 = XtCreateManagedWidget("Course   ", xmLabelWidgetClass, rowColumn_16, args, 0);
+    label_6 = XtCreateManagedWidget("Track    ", xmLabelWidgetClass, rowColumn_16, args, 0);
     label_7 = XtCreateManagedWidget("EPH      ", xmLabelWidgetClass, rowColumn_17, args, 0);
     label_8 = XtCreateManagedWidget("EPV      ", xmLabelWidgetClass, rowColumn_18, args, 0);
     label_9 = XtCreateManagedWidget("Climb    ", xmLabelWidgetClass, rowColumn_19, args, 0);
     label_10= XtCreateManagedWidget("Status   ", xmLabelWidgetClass, rowColumn_20, args, 0);
+    label_11= XtCreateManagedWidget("GPS type ", xmLabelWidgetClass, rowColumn_21, args, 0);
 
     /*@ -immediatetrans @*/
     XtSetArg(args[0], XmNcursorPositionVisible, False);
@@ -215,6 +220,8 @@ static void build_gui(Widget toplevel)
 				   rowColumn_19, args, 6);
     text_10 = XtCreateManagedWidget("text_10", xmTextFieldWidgetClass,
 				   rowColumn_20, args, 6);
+    text_11 = XtCreateManagedWidget("text_11", xmTextFieldWidgetClass,
+				   rowColumn_21, args, 6);
 
 #ifdef EXPLICIT_QUIT
     quitbutton = XtCreateManagedWidget("            Quit            ",
@@ -252,7 +259,6 @@ static void handle_time_out(XtPointer client_data UNUSED,
 static struct gps_data_t *gpsdata;
 static time_t timer;	/* time of last state change */
 static int state = 0;	/* or MODE_NO_FIX=1, MODE_2D=2, MODE_3D=3 */
-static int smoothing = 4;	/* # of transmitted sentences to smooth across */
 static XtAppContext app;
 static XtIntervalId timeout;
 static enum deg_str_type deg_type = deg_dd;
@@ -271,9 +277,8 @@ static void update_panel(struct gps_data_t *gpsdata,
 			 size_t len UNUSED, int level UNUSED)
 /* runs on each sentence */
 {
-    static int lfok = 0;
     unsigned int i;
-    int newstate, newtxt;
+    int newstate;
     XmString string[MAXCHANNELS+1];
     char s[128], *latlon, *sp;
 
@@ -287,12 +292,12 @@ static void update_panel(struct gps_data_t *gpsdata,
 	for (i = 0; i < MAXCHANNELS; i++) {
 	    if (i < (unsigned int)gpsdata->satellites) {
 		(void)snprintf(s, sizeof(s),  
-			       " %3d    %02d    %03d    %02d      %c", 
+			       " %3d    %2d    %3d    %2d      %c", 
 			       gpsdata->PRN[i],
 			       gpsdata->elevation[i], gpsdata->azimuth[i], 
 			       gpsdata->ss[i],	gpsdata->used[i] ? 'Y' : 'N');
 	    } else
-		(void)strcpy(s, "                  ");
+		(void)strlcpy(s, "                  ", 128);
 	    string[i+1] = XmStringCreateSimple(s);
 	}
 	XmListReplaceItemsPos(satellite_list, string, (int)sizeof(string), 1);
@@ -301,175 +306,172 @@ static void update_panel(struct gps_data_t *gpsdata,
     }
     /* here are the value fields */
     if (isnan(gpsdata->fix.time)==0) {
-	    (void)unix_to_iso8601(gpsdata->fix.time, s, (int)sizeof(s));
-	    newtxt = 1;
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_1, s);
-	if (gpsdata->fix.mode >= MODE_2D) {
+	(void)unix_to_iso8601(gpsdata->fix.time, s, (int)sizeof(s));
+	XmTextFieldSetString(text_1, s);
+    } else
+	XmTextFieldSetString(text_1, "n/a");
+    if (gpsdata->fix.mode >= MODE_2D) {
 	    latlon = deg_to_str(deg_type,  fabs(gpsdata->fix.latitude));
-	    newtxt = snprintf(s, sizeof(s), "%s %c", latlon, (gpsdata->fix.latitude < 0) ? 'S' : 'N');
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_2, s);
-	if (gpsdata->fix.mode >= MODE_2D) {
-	    latlon = deg_to_str(deg_type,  fabs(gpsdata->fix.longitude));
-	    newtxt = snprintf(s, sizeof(s), "%s %c", latlon, (gpsdata->fix.longitude < 0) ? 'W' : 'E');
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_3, s);
-	if (gpsdata->fix.mode == MODE_3D) {
-	    newtxt = snprintf(s, sizeof(s), "%f %s",gpsdata->fix.altitude*altunits->factor, altunits->legend);
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_4, s);
-	if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0) {
-	    newtxt = snprintf(s, sizeof(s), "%f %s", gpsdata->fix.speed*speedunits->factor, speedunits->legend);
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_5, s);
-	if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0) {
-	    newtxt = snprintf(s, sizeof(s), "%f degrees", gpsdata->fix.track);
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_6, s);
-	if (isnan(gpsdata->fix.eph)==0) {
-	    newtxt = snprintf(s, sizeof(s), "%f %s", gpsdata->fix.eph * altunits->factor, altunits->legend);
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_7, s);
-	if (isnan(gpsdata->fix.epv)==0) {
-	    newtxt = snprintf(s, sizeof(s), "%f %s", gpsdata->fix.epv * altunits->factor, altunits->legend);
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_8, s);
-	if (gpsdata->fix.mode == MODE_3D && isnan(gpsdata->fix.climb)==0) {
-	    newtxt = snprintf(s, sizeof(s), "%f %s/sec", gpsdata->fix.climb * altunits->factor, altunits->legend);
-	} else {
-	    newtxt = (lfok>0) ? 0 : ((void)strcpy(s, "n/a"), 1);
-	}
-	if (newtxt != 0) XmTextFieldSetString(text_9, s);
-
-	if (gpsdata->online == 0) {
-	    newstate = 0;
-	    (void)snprintf(s, sizeof(s), "OFFLINE");
-	    lfok = 0;
-	} else {
-	    newstate = gpsdata->fix.mode;
-	    switch (gpsdata->fix.mode) {
-	    case MODE_2D:
-		(void)snprintf(s, sizeof(s), "2D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
-		if (lfok>0) lfok--;
-		break;
-	    case MODE_3D:
-		(void)snprintf(s, sizeof(s), "3D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
-		lfok = smoothing;
-		break;
-	    default:
-		(void)snprintf(s, sizeof(s), "NO FIX");
-		lfok = 0;
-		break;
-	    }
-	}
-	if (newstate != state) {
-	    timer = time(NULL);
-	    state = newstate;
-	}
-	(void)snprintf(s+strlen(s), sizeof(s)-strlen(s),
-		       " (%d secs)", (int) (time(NULL) - timer));
-	XmTextFieldSetString(text_10, s);
-	draw_graphics(gpsdata);
-
-	XtRemoveTimeOut(timeout);
-	timeout = XtAppAddTimeOut(app, 2000, handle_time_out, NULL);
+	    (void)snprintf(s, sizeof(s), "%s %c", 
+			   latlon, (gpsdata->fix.latitude < 0) ? 'S' : 'N');
+	XmTextFieldSetString(text_2, s);
+    } else
+	XmTextFieldSetString(text_2, "n/a");
+    if (gpsdata->fix.mode >= MODE_2D) {
+	latlon = deg_to_str(deg_type,  fabs(gpsdata->fix.longitude));
+	(void)snprintf(s, sizeof(s), "%s %c", 
+		       latlon, (gpsdata->fix.longitude < 0) ? 'W' : 'E');
+	XmTextFieldSetString(text_3, s);
+    } else
+	XmTextFieldSetString(text_3, "n/a");
+    if (gpsdata->fix.mode == MODE_3D) {
+	(void)snprintf(s, sizeof(s), "%f %s",
+	       gpsdata->fix.altitude*altunits->factor, altunits->legend);
+	XmTextFieldSetString(text_4, s);
+    } else
+	XmTextFieldSetString(text_4, "n/a");
+    if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0) {
+	(void)snprintf(s, sizeof(s), "%f %s",
+	       gpsdata->fix.speed*speedunits->factor, speedunits->legend);
+	XmTextFieldSetString(text_5, s);
+    } else
+	XmTextFieldSetString(text_5, "n/a");
+    if (gpsdata->fix.mode >= MODE_2D && isnan(gpsdata->fix.track)==0) {
+	(void)snprintf(s, sizeof(s), "%f degrees", gpsdata->fix.track);
+	XmTextFieldSetString(text_6, s);
+    } else
+	XmTextFieldSetString(text_6, "n/a");
+    if (isnan(gpsdata->fix.eph)==0) {
+	(void)snprintf(s, sizeof(s), "%f %s", 
+		       gpsdata->fix.eph * altunits->factor, altunits->legend);
+	XmTextFieldSetString(text_7, s);
+    } else
+	XmTextFieldSetString(text_7, "n/a");
+    if (isnan(gpsdata->fix.epv)==0) {
+	(void)snprintf(s, sizeof(s), "%f %s", 
+		       gpsdata->fix.epv * altunits->factor, altunits->legend);
+	XmTextFieldSetString(text_8, s);
+    } else
+	XmTextFieldSetString(text_8, "n/a");
+    if (gpsdata->fix.mode == MODE_3D && isnan(gpsdata->fix.climb)==0) {
+	(void)snprintf(s, sizeof(s), "%f %s/sec", 
+		       gpsdata->fix.climb*altunits->factor, altunits->legend);
+	XmTextFieldSetString(text_9, s);
+    } else
+	XmTextFieldSetString(text_9, "n/a");
+    if (gpsdata->set & DEVICEID_SET) {
+	(void)strlcpy(s, gpsdata->gps_id, sizeof(s));
+	XmTextFieldSetString(text_11, s);
     }
-
-    static char *get_resource(Widget w, char *name, char *default_value)
-    {
-      XtResource xtr;
-      char *value = NULL;
-
-      /*@ -observertrans -statictrans -immediatetrans -compdestroy @*/
-      xtr.resource_name = name;
-      xtr.resource_class = "AnyClass";
-      xtr.resource_type = XmRString;
-      xtr.resource_size = (Cardinal)sizeof(String);
-      xtr.resource_offset = 0;
-      xtr.default_type = XmRImmediate;
-      xtr.default_addr = default_value;
-      XtGetApplicationResources(w, &value, &xtr, 1, NULL, 0);
-      if (value) return value;
-      /*@ +observertrans +statictrans +immediatetrans +compdestroy @*/
-      /*@i@*/return default_value;
-    }
-
-    /*@ -mustfreefresh @*/
-    int main(int argc, char *argv[])
-    {
-	int option;
-	char *arg = NULL, *colon1, *colon2, *device = NULL, *server = NULL, *port = DEFAULT_GPSD_PORT;
-	char *su, *au;
-	char *err_str = NULL;
-
-	/*@ -onlytrans */
-	toplevel = XtVaAppInitialize(&app, "xgps", 
-				   options, XtNumber(options), 
-				   &argc,argv, fallback_resources,NULL);
-
-	/*@ +onlytrans */
-	su = get_resource(toplevel, "speedunits", "mph");
-	for (speedunits = speedtable; 
-	     speedunits < speedtable + sizeof(speedtable)/sizeof(speedtable[0]);
-	     speedunits++)
-	    if (strcmp(speedunits->legend, su) == 0)
-		goto speedunits_ok;
-	speedunits = speedtable;
-	(void)fprintf(stderr, "xgps: unknown speed unit, defaulting to %s\n", speedunits->legend);
-    speedunits_ok:;
-
-	au = get_resource(toplevel, "altunits",   "feet");
-	for (altunits = alttable; 
-	     altunits < alttable + sizeof(alttable)/sizeof(alttable[0]);
-	     altunits++)
-	    if (strcmp(altunits->legend, au) == 0)
-		goto altunits_ok;
-	altunits = alttable;
-	(void)fprintf(stderr, "xgps: unknown altitude unit, defaulting to %s\n", altunits->legend);
-    altunits_ok:;
-
-	while ((option = getopt(argc, argv, "hl:s:V")) != -1) {
-	    switch (option) {
-	    case 'V':
-		(void)printf("xgps %s\n", VERSION);
-		exit(0);
-	    case 'l':
-		switch ( optarg[0] ) {
-		case 'd':
-		    deg_type = deg_dd;
-		    continue;
-		case 'm':
-		    deg_type = deg_ddmm;
-		    continue;
-		case 's':
-		    deg_type = deg_ddmmss;
-		    continue;
-		default:
-		    (void)fprintf(stderr, "Unknown -l argument: %s\n", optarg);
-		    /*@ -casebreak @*/
-		}
-	case 's':
-	    smoothing = atoi(optarg);
+    if (gpsdata->online == 0) {
+	newstate = 0;
+	(void)snprintf(s, sizeof(s), "OFFLINE");
+    } else {
+	newstate = gpsdata->fix.mode;
+	switch (gpsdata->fix.mode) {
+	case MODE_2D:
+	    (void)snprintf(s, sizeof(s), "2D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
 	    break;
+	case MODE_3D:
+	    (void)snprintf(s, sizeof(s), "3D %sFIX",(gpsdata->status==STATUS_DGPS_FIX)?"DIFF ":"");
+	    break;
+	default:
+	    (void)snprintf(s, sizeof(s), "NO FIX");
+	    break;
+	}
+    }
+    if (newstate != state) {
+	timer = time(NULL);
+	state = newstate;
+    }
+    (void)snprintf(s+strlen(s), sizeof(s)-strlen(s),
+		   " (%d secs)", (int) (time(NULL) - timer));
+    XmTextFieldSetString(text_10, s);
+    draw_graphics(gpsdata);
+
+    XtRemoveTimeOut(timeout);
+    timeout = XtAppAddTimeOut(app, 2000, handle_time_out, NULL);
+}
+
+static char *get_resource(Widget w, char *name, char *default_value)
+{
+    XtResource xtr;
+    char *value = NULL;
+
+    /*@ -observertrans -statictrans -immediatetrans -compdestroy @*/
+    xtr.resource_name = name;
+    xtr.resource_class = "AnyClass";
+    xtr.resource_type = XmRString;
+    xtr.resource_size = (Cardinal)sizeof(String);
+    xtr.resource_offset = 0;
+    xtr.default_type = XmRImmediate;
+    xtr.default_addr = default_value;
+    XtGetApplicationResources(w, &value, &xtr, 1, NULL, 0);
+    if (value) return value;
+    /*@ +observertrans +statictrans +immediatetrans +compdestroy @*/
+    /*@i@*/return default_value;
+}
+
+/*@ -mustfreefresh @*/
+int main(int argc, char *argv[])
+{
+    int option;
+    char *arg = NULL, *colon1, *colon2, *device = NULL, *server = NULL, *port = DEFAULT_GPSD_PORT;
+    char *su, *au;
+    char *err_str = NULL;
+    bool jitteropt = false;
+
+    /*@ -onlytrans */
+    toplevel = XtVaAppInitialize(&app, "xgps", 
+				 options, XtNumber(options), 
+				 &argc,argv, fallback_resources,NULL);
+
+    /*@ +onlytrans */
+    su = get_resource(toplevel, "speedunits", "mph");
+    for (speedunits = speedtable; 
+	 speedunits < speedtable + sizeof(speedtable)/sizeof(speedtable[0]);
+	 speedunits++)
+	if (strcmp(speedunits->legend, su) == 0)
+	    goto speedunits_ok;
+    speedunits = speedtable;
+    (void)fprintf(stderr, "xgps: unknown speed unit, defaulting to %s\n", speedunits->legend);
+speedunits_ok:;
+
+    au = get_resource(toplevel, "altunits",   "feet");
+    for (altunits = alttable; 
+	 altunits < alttable + sizeof(alttable)/sizeof(alttable[0]);
+	 altunits++)
+	if (strcmp(altunits->legend, au) == 0)
+	    goto altunits_ok;
+    altunits = alttable;
+    (void)fprintf(stderr, "xgps: unknown altitude unit, defaulting to %s\n", altunits->legend);
+altunits_ok:;
+
+    while ((option = getopt(argc, argv, "hjl:V")) != -1) {
+	switch (option) {
+	case 'V':
+	    (void)printf("xgps %s\n", VERSION);
+	    exit(0);
+	case 'j':
+	    jitteropt = true;
+	    continue;
+	case 'l':
+	    switch ( optarg[0] ) {
+	    case 'd':
+		deg_type = deg_dd;
+		continue;
+	    case 'm':
+		deg_type = deg_ddmm;
+		continue;
+	    case 's':
+		deg_type = deg_ddmmss;
+		continue;
+	    default:
+		(void)fprintf(stderr, "Unknown -l argument: %s\n", optarg);
+		/*@ -casebreak @*/
+	    }
 	case 'h': default:
-	    (void)fputs("usage:  xgps [-hV] [-speedunits {mph,kph,knots}] [-altunits {ft,meters}] [-l {d|m|s}] [-s smoothing] [server[:port:[device]]]\n", stderr);
+	    (void)fputs("usage:  xgps [-hjV] [-speedunits {mph,kph,knots}] [-altunits {ft,meters}] [-l {d|m|s}] [server[:port:[device]]]\n", stderr);
 	    exit(1);
 	}
     }
@@ -521,21 +523,16 @@ static void update_panel(struct gps_data_t *gpsdata,
 
     gps_set_raw_hook(gpsdata, update_panel);
 
-    if (device) {
-	char *channelcmd = (char *)malloc(strlen(device)+3);
+    if (jitteropt)
+	(void)gps_query(gpsdata, "J=1");
 
-	if (channelcmd) {
-	    /*@i1@*/(void)strcpy(channelcmd, "F=");
-	    (void)strcpy(channelcmd+2, device);
-	    (void)gps_query(gpsdata, channelcmd);
-	    (void)free(channelcmd);
-	}
-    }
-	
-    (void)gps_query(gpsdata, "w+x\n");
+    if (device)
+	(void)gps_query(gpsdata, "F=%s", device);
+
+    (void)gps_query(gpsdata, "w+x");
 
     (void)XtAppAddInput(app, gpsdata->gps_fd, 
-		  (XtPointer)XtInputReadMask, handle_input, NULL);
+			(XtPointer)XtInputReadMask, handle_input, NULL);
     (void)XtAppMainLoop(app);
 
     (void)gps_close(gpsdata);
